"""
NPCID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 06:54 UTC
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class NPCID:
    Count = 697

    class BasicNPCPortrait(IntEnum):
        NegativeIDCount = -66
        BigHornetStingy = -65
        LittleHornetStingy = -64
        BigHornetSpikey = -63
        LittleHornetSpikey = -62
        BigHornetLeafy = -61
        LittleHornetLeafy = -60
        BigHornetHoney = -59
        LittleHornetHoney = -58
        BigHornetFatty = -57
        LittleHornetFatty = -56
        BigRainZombie = -55
        SmallRainZombie = -54
        BigPantlessSkeleton = -53
        SmallPantlessSkeleton = -52
        BigMisassembledSkeleton = -51
        SmallMisassembledSkeleton = -50
        BigHeadacheSkeleton = -49
        SmallHeadacheSkeleton = -48
        BigSkeleton = -47
        SmallSkeleton = -46
        BigFemaleZombie = -45
        SmallFemaleZombie = -44
        DemonEye2 = -43
        PurpleEye2 = -42
        GreenEye2 = -41
        DialatedEye2 = -40
        SleepyEye2 = -39
        CataractEye2 = -38
        BigTwiggyZombie = -37
        SmallTwiggyZombie = -36
        BigSwampZombie = -35
        SmallSwampZombie = -34
        BigSlimedZombie = -33
        SmallSlimedZombie = -32
        BigPincushionZombie = -31
        SmallPincushionZombie = -30
        BigBaldZombie = -29
        SmallBaldZombie = -28
        BigZombie = -27
        SmallZombie = -26
        BigCrimslime = -25
        LittleCrimslime = -24
        BigCrimera = -23
        LittleCrimera = -22
        GiantMossHornet = -21
        BigMossHornet = -20
        LittleMossHornet = -19
        TinyMossHornet = -18
        BigStinger = -17
        LittleStinger = -16
        HeavySkeleton = -15
        BigBoned = -14
        ShortBones = -13
        BigEater = -12
        LittleEater = -11
        JungleSlime = -10
        YellowSlime = -9
        RedSlime = -8
        PurpleSlime = -7
        BlackSlime = -6
        BabySlime = -5
        Pinky = -4
        GreenSlime = -3
        Slimer2 = -2
        Slimeling = -1
        NoneValue = 0
        BlueSlime = 1
        DemonEye = 2
        Zombie = 3
        EyeofCthulhu = 4
        ServantofCthulhu = 5
        EaterofSouls = 6
        DevourerHead = 7
        DevourerBody = 8
        DevourerTail = 9
        GiantWormHead = 10
        GiantWormBody = 11
        GiantWormTail = 12
        EaterofWorldsHead = 13
        EaterofWorldsBody = 14
        EaterofWorldsTail = 15
        MotherSlime = 16
        Merchant = 17
        Nurse = 18
        ArmsDealer = 19
        Dryad = 20
        Skeleton = 21
        Guide = 22
        MeteorHead = 23
        FireImp = 24
        BurningSphere = 25
        GoblinPeon = 26
        GoblinThief = 27
        GoblinWarrior = 28
        GoblinSorcerer = 29
        ChaosBall = 30
        AngryBones = 31
        DarkCaster = 32
        WaterSphere = 33
        CursedSkull = 34
        SkeletronHead = 35
        SkeletronHand = 36
        OldMan = 37
        Demolitionist = 38
        BoneSerpentHead = 39
        BoneSerpentBody = 40
        BoneSerpentTail = 41
        Hornet = 42
        ManEater = 43
        UndeadMiner = 44
        Tim = 45
        Bunny = 46
        CorruptBunny = 47
        Harpy = 48
        CaveBat = 49
        KingSlime = 50
        JungleBat = 51
        DoctorBones = 52
        TheGroom = 53
        Clothier = 54
        Goldfish = 55
        Snatcher = 56
        CorruptGoldfish = 57
        Piranha = 58
        LavaSlime = 59
        Hellbat = 60
        Vulture = 61
        Demon = 62
        BlueJellyfish = 63
        PinkJellyfish = 64
        Shark = 65
        VoodooDemon = 66
        Crab = 67
        DungeonGuardian = 68
        Antlion = 69
        SpikeBall = 70
        DungeonSlime = 71
        BlazingWheel = 72
        GoblinScout = 73
        Bird = 74
        Pixie = 75
        None2 = 76
        ArmoredSkeleton = 77
        Mummy = 78
        DarkMummy = 79
        LightMummy = 80
        CorruptSlime = 81
        Wraith = 82
        CursedHammer = 83
        EnchantedSword = 84
        Mimic = 85
        Unicorn = 86
        WyvernHead = 87
        WyvernLegs = 88
        WyvernBody = 89
        WyvernBody2 = 90
        WyvernBody3 = 91
        WyvernTail = 92
        GiantBat = 93
        Corruptor = 94
        DiggerHead = 95
        DiggerBody = 96
        DiggerTail = 97
        SeekerHead = 98
        SeekerBody = 99
        SeekerTail = 100
        Clinger = 101
        AnglerFish = 102
        GreenJellyfish = 103
        Werewolf = 104
        BoundGoblin = 105
        BoundWizard = 106
        GoblinTinkerer = 107
        Wizard = 108
        Clown = 109
        SkeletonArcher = 110
        GoblinArcher = 111
        VileSpit = 112
        WallofFlesh = 113
        WallofFleshEye = 114
        TheHungry = 115
        TheHungryII = 116
        LeechHead = 117
        LeechBody = 118
        LeechTail = 119
        ChaosElemental = 120
        Slimer = 121
        Gastropod = 122
        BoundMechanic = 123
        Mechanic = 124
        Retinazer = 125
        Spazmatism = 126
        SkeletronPrime = 127
        PrimeCannon = 128
        PrimeSaw = 129
        PrimeVice = 130
        PrimeLaser = 131
        BaldZombie = 132
        WanderingEye = 133
        TheDestroyer = 134
        TheDestroyerBody = 135
        TheDestroyerTail = 136
        IlluminantBat = 137
        IlluminantSlime = 138
        Probe = 139
        PossessedArmor = 140
        ToxicSludge = 141
        SantaClaus = 142
        SnowmanGangsta = 143
        MisterStabby = 144
        SnowBalla = 145
        None3 = 146
        IceSlime = 147
        Penguin = 148
        PenguinBlack = 149
        IceBat = 150
        Lavabat = 151
        GiantFlyingFox = 152
        GiantTortoise = 153
        IceTortoise = 154
        Wolf = 155
        RedDevil = 156
        Arapaima = 157
        VampireBat = 158
        Vampire = 159
        Truffle = 160
        ZombieEskimo = 161
        Frankenstein = 162
        BlackRecluse = 163
        WallCreeper = 164
        WallCreeperWall = 165
        SwampThing = 166
        UndeadViking = 167
        CorruptPenguin = 168
        IceElemental = 169
        PigronCorruption = 170
        PigronHallow = 171
        RuneWizard = 172
        Crimera = 173
        Herpling = 174
        AngryTrapper = 175
        MossHornet = 176
        Derpling = 177
        Steampunker = 178
        CrimsonAxe = 179
        PigronCrimson = 180
        FaceMonster = 181
        FloatyGross = 182
        Crimslime = 183
        SpikedIceSlime = 184
        SnowFlinx = 185
        PincushionZombie = 186
        SlimedZombie = 187
        SwampZombie = 188
        TwiggyZombie = 189
        CataractEye = 190
        SleepyEye = 191
        DialatedEye = 192
        GreenEye = 193
        PurpleEye = 194
        LostGirl = 195
        Nymph = 196
        ArmoredViking = 197
        Lihzahrd = 198
        LihzahrdCrawler = 199
        FemaleZombie = 200
        HeadacheSkeleton = 201
        MisassembledSkeleton = 202
        PantlessSkeleton = 203
        SpikedJungleSlime = 204
        Moth = 205
        IcyMerman = 206
        DyeTrader = 207
        PartyGirl = 208
        Cyborg = 209
        Bee = 210
        BeeSmall = 211
        PirateDeckhand = 212
        PirateCorsair = 213
        PirateDeadeye = 214
        PirateCrossbower = 215
        PirateCaptain = 216
        CochinealBeetle = 217
        CyanBeetle = 218
        LacBeetle = 219
        SeaSnail = 220
        Squid = 221
        QueenBee = 222
        ZombieRaincoat = 223
        FlyingFish = 224
        UmbrellaSlime = 225
        FlyingSnake = 226
        Painter = 227
        WitchDoctor = 228
        Pirate = 229
        GoldfishWalker = 230
        HornetFatty = 231
        HornetHoney = 232
        HornetLeafy = 233
        HornetSpikey = 234
        HornetStingy = 235
        JungleCreeper = 236
        JungleCreeperWall = 237
        BlackRecluseWall = 238
        BloodCrawler = 239
        BloodCrawlerWall = 240
        BloodFeeder = 241
        BloodJelly = 242
        IceGolem = 243
        RainbowSlime = 244
        Golem = 245
        GolemHead = 246
        GolemFistLeft = 247
        GolemFistRight = 248
        GolemHeadFree = 249
        AngryNimbus = 250
        Eyezor = 251
        Parrot = 252
        Reaper = 253
        ZombieMushroom = 254
        ZombieMushroomHat = 255
        FungoFish = 256
        AnomuraFungus = 257
        MushiLadybug = 258
        FungiBulb = 259
        GiantFungiBulb = 260
        FungiSpore = 261
        Plantera = 262
        PlanterasHook = 263
        PlanterasTentacle = 264
        Spore = 265
        BrainofCthulhu = 266
        Creeper = 267
        IchorSticker = 268
        RustyArmoredBonesAxe = 269
        RustyArmoredBonesFlail = 270
        RustyArmoredBonesSword = 271
        RustyArmoredBonesSwordNoArmor = 272
        BlueArmoredBones = 273
        BlueArmoredBonesMace = 274
        BlueArmoredBonesNoPants = 275
        BlueArmoredBonesSword = 276
        HellArmoredBones = 277
        HellArmoredBonesSpikeShield = 278
        HellArmoredBonesMace = 279
        HellArmoredBonesSword = 280
        RaggedCaster = 281
        RaggedCasterOpenCoat = 282
        Necromancer = 283
        NecromancerArmored = 284
        DiabolistRed = 285
        DiabolistWhite = 286
        BoneLee = 287
        DungeonSpirit = 288
        GiantCursedSkull = 289
        Paladin = 290
        SkeletonSniper = 291
        TacticalSkeleton = 292
        SkeletonCommando = 293
        AngryBonesBig = 294
        AngryBonesBigMuscle = 295
        AngryBonesBigHelmet = 296
        BirdBlue = 297
        BirdRed = 298
        Squirrel = 299
        Mouse = 300
        Raven = 301
        SlimeMasked = 302
        BunnySlimed = 303
        HoppinJack = 304
        Scarecrow1 = 305
        Scarecrow2 = 306
        Scarecrow3 = 307
        Scarecrow4 = 308
        Scarecrow5 = 309
        Scarecrow6 = 310
        Scarecrow7 = 311
        Scarecrow8 = 312
        Scarecrow9 = 313
        Scarecrow10 = 314
        HeadlessHorseman = 315
        Ghost = 316
        DemonEyeOwl = 317
        DemonEyeSpaceship = 318
        ZombieDoctor = 319
        ZombieSuperman = 320
        ZombiePixie = 321
        SkeletonTopHat = 322
        SkeletonAstonaut = 323
        SkeletonAlien = 324
        MourningWood = 325
        Splinterling = 326
        Pumpking = 327
        PumpkingBlade = 328
        Hellhound = 329
        Poltergeist = 330
        ZombieXmas = 331
        ZombieSweater = 332
        SlimeRibbonWhite = 333
        SlimeRibbonYellow = 334
        SlimeRibbonGreen = 335
        SlimeRibbonRed = 336
        BunnyXmas = 337
        ZombieElf = 338
        ZombieElfBeard = 339
        ZombieElfGirl = 340
        PresentMimic = 341
        GingerbreadMan = 342
        Yeti = 343
        Everscream = 344
        IceQueen = 345
        SantaNK1 = 346
        ElfCopter = 347
        Nutcracker = 348
        NutcrackerSpinning = 349
        ElfArcher = 350
        Krampus = 351
        Flocko = 352
        Stylist = 353
        WebbedStylist = 354
        Firefly = 355
        Butterfly = 356
        Worm = 357
        LightningBug = 358
        Snail = 359
        GlowingSnail = 360
        Frog = 361
        Duck = 362
        Duck2 = 363
        DuckWhite = 364
        DuckWhite2 = 365
        ScorpionBlack = 366
        Scorpion = 367
        TravellingMerchant = 368
        Angler = 369
        DukeFishron = 370
        DetonatingBubble = 371
        Sharkron = 372
        Sharkron2 = 373
        TruffleWorm = 374
        TruffleWormDigger = 375
        SleepingAngler = 376
        Grasshopper = 377
        ChatteringTeethBomb = 378
        CultistArcherBlue = 379
        CultistArcherWhite = 380
        BrainScrambler = 381
        RayGunner = 382
        MartianOfficer = 383
        ForceBubble = 384
        GrayGrunt = 385
        MartianEngineer = 386
        MartianTurret = 387
        MartianDrone = 388
        GigaZapper = 389
        ScutlixRider = 390
        Scutlix = 391
        MartianSaucer = 392
        MartianSaucerTurret = 393
        MartianSaucerCannon = 394
        MartianSaucerCore = 395
        MoonLordHead = 396
        MoonLordHand = 397
        MoonLordCore = 398
        MartianProbe = 399
        MoonLordFreeEye = 400
        MoonLordLeechBlob = 401
        StardustWormHead = 402
        StardustWormBody = 403
        StardustWormTail = 404
        StardustCellBig = 405
        StardustCellSmall = 406
        StardustJellyfishBig = 407
        StardustJellyfishSmall = 408
        StardustSpiderBig = 409
        StardustSpiderSmall = 410
        StardustSoldier = 411
        SolarCrawltipedeHead = 412
        SolarCrawltipedeBody = 413
        SolarCrawltipedeTail = 414
        SolarDrakomire = 415
        SolarDrakomireRider = 416
        SolarSroller = 417
        SolarCorite = 418
        SolarSolenian = 419
        NebulaBrain = 420
        NebulaHeadcrab = 421
        LunarTowerVortex = 422
        NebulaBeast = 423
        NebulaSoldier = 424
        VortexRifleman = 425
        VortexHornetQueen = 426
        VortexHornet = 427
        VortexLarva = 428
        VortexSoldier = 429
        ArmedZombie = 430
        ArmedZombieEskimo = 431
        ArmedZombiePincussion = 432
        ArmedZombieSlimed = 433
        ArmedZombieSwamp = 434
        ArmedZombieTwiggy = 435
        ArmedZombieCenx = 436
        CultistTablet = 437
        CultistDevote = 438
        CultistBoss = 439
        CultistBossClone = 440
        TaxCollector = 441
        GoldBird = 442
        GoldBunny = 443
        GoldButterfly = 444
        GoldFrog = 445
        GoldGrasshopper = 446
        GoldMouse = 447
        GoldWorm = 448
        BoneThrowingSkeleton = 449
        BoneThrowingSkeleton2 = 450
        BoneThrowingSkeleton3 = 451
        BoneThrowingSkeleton4 = 452
        SkeletonMerchant = 453
        CultistDragonHead = 454
        CultistDragonBody1 = 455
        CultistDragonBody2 = 456
        CultistDragonBody3 = 457
        CultistDragonBody4 = 458
        CultistDragonTail = 459
        Butcher = 460
        CreatureFromTheDeep = 461
        Fritz = 462
        Nailhead = 463
        CrimsonBunny = 464
        CrimsonGoldfish = 465
        Psycho = 466
        DeadlySphere = 467
        DrManFly = 468
        ThePossessed = 469
        CrimsonPenguin = 470
        GoblinSummoner = 471
        ShadowFlameApparition = 472
        BigMimicCorruption = 473
        BigMimicCrimson = 474
        BigMimicHallow = 475
        BigMimicJungle = 476
        Mothron = 477
        MothronEgg = 478
        MothronSpawn = 479
        Medusa = 480
        GreekSkeleton = 481
        GraniteGolem = 482
        GraniteFlyer = 483
        EnchantedNightcrawler = 484
        Grubby = 485
        Sluggy = 486
        Buggy = 487
        TargetDummy = 488
        BloodZombie = 489
        Drippler = 490
        PirateShip = 491
        PirateShipCannon = 492
        LunarTowerStardust = 493
        Crawdad = 494
        Crawdad2 = 495
        GiantShelly = 496
        GiantShelly2 = 497
        Salamander = 498
        Salamander2 = 499
        Salamander3 = 500
        Salamander4 = 501
        Salamander5 = 502
        Salamander6 = 503
        Salamander7 = 504
        Salamander8 = 505
        Salamander9 = 506
        LunarTowerNebula = 507
        GiantWalkingAntlion = 508
        GiantFlyingAntlion = 509
        DuneSplicerHead = 510
        DuneSplicerBody = 511
        DuneSplicerTail = 512
        TombCrawlerHead = 513
        TombCrawlerBody = 514
        TombCrawlerTail = 515
        SolarFlare = 516
        LunarTowerSolar = 517
        SolarSpearman = 518
        SolarGoop = 519
        MartianWalker = 520
        AncientCultistSquidhead = 521
        AncientLight = 522
        AncientDoom = 523
        DesertGhoul = 524
        DesertGhoulCorruption = 525
        DesertGhoulCrimson = 526
        DesertGhoulHallow = 527
        DesertLamiaLight = 528
        DesertLamiaDark = 529
        DesertScorpionWalk = 530
        DesertScorpionWall = 531
        DesertBeast = 532
        DesertDjinn = 533
        DemonTaxCollector = 534
        SlimeSpiked = 535
        TheBride = 536
        SandSlime = 537
        SquirrelRed = 538
        SquirrelGold = 539
        PartyBunny = 540
        SandElemental = 541
        SandShark = 542
        SandsharkCorrupt = 543
        SandsharkCrimson = 544
        SandsharkHallow = 545
        Tumbleweed = 546
        DD2AttackerTest = 547
        DD2EterniaCrystal = 548
        DD2LanePortal = 549
        DD2Bartender = 550
        DD2Betsy = 551
        DD2GoblinT1 = 552
        DD2GoblinT2 = 553
        DD2GoblinT3 = 554
        DD2GoblinBomberT1 = 555
        DD2GoblinBomberT2 = 556
        DD2GoblinBomberT3 = 557
        DD2WyvernT1 = 558
        DD2WyvernT2 = 559
        DD2WyvernT3 = 560
        DD2JavelinstT1 = 561
        DD2JavelinstT2 = 562
        DD2JavelinstT3 = 563
        DD2DarkMageT1 = 564
        DD2DarkMageT3 = 565
        DD2SkeletonT1 = 566
        DD2SkeletonT3 = 567
        DD2WitherBeastT2 = 568
        DD2WitherBeastT3 = 569
        DD2DrakinT2 = 570
        DD2DrakinT3 = 571
        DD2KoboldWalkerT2 = 572
        DD2KoboldWalkerT3 = 573
        DD2KoboldFlyerT2 = 574
        DD2KoboldFlyerT3 = 575
        DD2OgreT2 = 576
        DD2OgreT3 = 577
        DD2LightningBugT3 = 578
        BartenderUnconscious = 579
        WalkingAntlion = 580
        FlyingAntlion = 581
        LarvaeAntlion = 582
        FairyCritterPink = 583
        FairyCritterGreen = 584
        FairyCritterBlue = 585
        ZombieMerman = 586
        EyeballFlyingFish = 587
        Golfer = 588
        GolferRescue = 589
        TorchZombie = 590
        ArmedTorchZombie = 591
        GoldGoldfish = 592
        GoldGoldfishWalker = 593
        WindyBalloon = 594
        BlackDragonfly = 595
        BlueDragonfly = 596
        GreenDragonfly = 597
        OrangeDragonfly = 598
        RedDragonfly = 599
        YellowDragonfly = 600
        GoldDragonfly = 601
        Seagull = 602
        Seagull2 = 603
        LadyBug = 604
        GoldLadyBug = 605
        Maggot = 606
        Pupfish = 607
        Grebe = 608
        Grebe2 = 609
        Rat = 610
        Owl = 611
        WaterStrider = 612
        GoldWaterStrider = 613
        ExplosiveBunny = 614
        Dolphin = 615
        Turtle = 616
        TurtleJungle = 617
        BloodNautilus = 618
        BloodSquid = 619
        GoblinShark = 620
        BloodEelHead = 621
        BloodEelBody = 622
        BloodEelTail = 623
        Gnome = 624
        SeaTurtle = 625
        Seahorse = 626
        GoldSeahorse = 627
        Dandelion = 628
        IceMimic = 629
        BloodMummy = 630
        RockGolem = 631
        MaggotZombie = 632
        BestiaryGirl = 633
        SporeBat = 634
        SporeSkeleton = 635
        HallowBoss = 636
        TownCat = 637
        TownDog = 638
        GemSquirrelAmethyst = 639
        GemSquirrelTopaz = 640
        GemSquirrelSapphire = 641
        GemSquirrelEmerald = 642
        GemSquirrelRuby = 643
        GemSquirrelDiamond = 644
        GemSquirrelAmber = 645
        GemBunnyAmethyst = 646
        GemBunnyTopaz = 647
        GemBunnySapphire = 648
        GemBunnyEmerald = 649
        GemBunnyRuby = 650
        GemBunnyDiamond = 651
        GemBunnyAmber = 652
        HellButterfly = 653
        Lavafly = 654
        MagmaSnail = 655
        TownBunny = 656
        QueenSlimeBoss = 657
        QueenSlimeMinionBlue = 658
        QueenSlimeMinionPink = 659
        QueenSlimeMinionPurple = 660
        EmpressButterfly = 661
        PirateGhost = 662
        Princess = 663
        TorchGod = 664
        ChaosBallTim = 665
        VileSpitEaterOfWorlds = 666
        GoldenSlime = 667
        Deerclops = 668
        Stinkbug = 669
        TownSlimeBlue = 670
        ScarletMacaw = 671
        BlueMacaw = 672
        Toucan = 673
        YellowCockatiel = 674
        GrayCockatiel = 675
        ShimmerSlime = 676
        Shimmerfly = 677
        TownSlimeGreen = 678
        TownSlimeOld = 679
        TownSlimePurple = 680
        TownSlimeRainbow = 681
        TownSlimeRed = 682
        TownSlimeYellow = 683
        TownSlimeCopper = 684
        BoundTownSlimeOld = 685
        BoundTownSlimePurple = 686
        BoundTownSlimeYellow = 687
        Pufferfish = 688
        OwlMimic = 689
        StatueMimic = 690
        MossZombie = 691
        Orca = 692
        LibrarianSkeleton = 693
        WaterBoltMimic = 694
        PalworldCattivaDistressed = 695
        PalworldFoxsparksDistressed = 696
        Count = 697

    class NPCPortraitSelector(IntEnum):
        Poisoned = 20
        OnFire = 24
        Bleeding = 30
        Confused = 31
        CursedInferno = 39
        Frostburn = 44
        Ichor = 69
        Venom = 70
        ShadowFlame = 153
        BoneJavelin = 169
        Daybreak = 189
        BetsysCurse = 203
        Oiled = 204
        OnFire3 = 323
        Frostburn2 = 324
        TentacleSpike = 337
        BloodButcherer = 344
        Hemorrhage = 375


class NPCIDSets:
    class Main:
        factory = SetFactory(NPCID.Count)
        SpawnOnPlayerCanSpawnInMidairOnSkyblock = factory.create_bool_set(False, 4, 266, 222, 125, 126, 127, 262, 551)
        DontDropDungeonKeysOrSouls = factory.create_bool_set(False, 23)
        DontDoHardmodeScaling = factory.create_bool_set(False, 5, 13, 14, 15, 267, 113, 114, 115, 116, 117, 118, 119, 658, 659, 660, 400, 522)
        ReflectStarShotsInForTheWorthy = factory.create_bool_set(False, 4, 5, 13, 14, 15, 266, 267, 35, 36, 113, 114, 115, 116, 117, 118, 119, 125, 126, 134, 135, 136, 139, 127, 128, 131, 129, 130, 262, 263, 264, 245, 247, 248, 246, 249, 398, 400, 397, 396, 401)
        IsTownPet = factory.create_bool_set(False, 637, 638, 656, 670, 678, 679, 680, 681, 682, 683, 684)
        IsTownSlime = factory.create_bool_set(False, 670, 678, 679, 680, 681, 682, 683, 684)
        CanConvertIntoCopperSlimeTownNPC = factory.create_bool_set(False, 1, 302, 335, 336, 333, 334)
        IsGoldCritter = factory.create_bool_set(False, 442, 443, 444, 445, 446, 447, 448, 539, 592, 593, 601, 605, 613, 627)
        ZappingJellyfish = factory.create_bool_set(False, 63, 64, 103, 242)
        CantTakeLunchMoney = factory.create_bool_set(False, 394, 393, 392, 690, 492, 491, 662, 384, 478, 535, 658, 659, 660, 128, 131, 129, 130, 139, 267, 247, 248, 246, 249, 245, 409, 410, 397, 396, 401, 400, 440, 68, 534, 316)
        TrailingMode = factory.create_int_set(-1, 439, 0, 440, 0, 370, 1, 372, 1, 373, 1, 396, 1, 400, 1, 401, 1, 473, 2, 474, 2, 475, 2, 476, 2, 4, 3, 471, 3, 477, 3, 479, 3, 120, 4, 137, 4, 138, 4, 94, 5, 125, 6, 126, 6, 127, 6, 128, 6, 129, 6, 130, 6, 131, 6, 139, 6, 140, 6, 407, 6, 420, 6, 425, 6, 427, 6, 426, 6, 581, 6, 516, 6, 542, 6, 543, 6, 544, 6, 545, 6, 402, 7, 417, 7, 419, 7, 418, 7, 574, 7, 575, 7, 519, 7, 521, 7, 522, 7, 546, 7, 558, 7, 559, 7, 560, 7, 551, 7, 620, 7, 657, 6, 636, 7, 677, 7, 685, 7)
        IsDragonfly = factory.create_bool_set(False, 595, 596, 597, 598, 599, 600, 601)
        BelongsToInvasionOldOnesArmy = factory.create_bool_set(False, 552, 553, 554, 561, 562, 563, 555, 556, 557, 558, 559, 560, 576, 577, 568, 569, 566, 567, 570, 571, 572, 573, 548, 549, 564, 565, 574, 575, 551, 578)
        TeleportationImmune = factory.create_bool_set(False, 552, 553, 554, 561, 562, 563, 555, 556, 557, 558, 559, 560, 576, 577, 568, 569, 566, 567, 570, 571, 572, 573, 548, 549, 564, 565, 574, 575, 551, 578)
        UsesNewTargetting = factory.create_bool_set(False, 547, 552, 553, 554, 561, 562, 563, 555, 556, 557, 558, 559, 560, 576, 577, 568, 569, 566, 567, 570, 571, 572, 573, 564, 565, 574, 575, 551, 578, 210, 211, 620, 668)
        BirdThatCanPoop = factory.create_bool_set(False, 74, 297, 298, 442, 363, 365, 603, 609, 671, 672, 673, 674, 675)
        CritterThatCanTurnOnPlayers = factory.create_bool_set(False, 645, 639, 644, 642, 643, 641, 640, 299, 539, 538)
        TakesDamageFromHostilesWithoutBeingFriendly = factory.create_bool_set(False, 46, 55, 74, 148, 149, 230, 297, 298, 299, 303, 355, 356, 358, 359, 360, 361, 362, 363, 364, 365, 366, 367, 377, 357, 374, 442, 443, 444, 445, 446, 448, 538, 539, 337, 540, 484, 485, 486, 487, 592, 593, 595, 596, 597, 598, 599, 600, 601, 602, 603, 604, 605, 606, 607, 608, 609, 611, 689, 612, 613, 614, 615, 616, 617, 625, 626, 627, 639, 640, 641, 642, 643, 644, 645, 646, 647, 648, 649, 650, 651, 652, 653, 654, 655, 583, 584, 585, 669, 671, 672, 673, 674, 675, 677, 687, 688)
        AllNPCs = factory.create_bool_set(True)
        HurtingBees = factory.create_bool_set(False, 210, 211)
        CanBeHurtByBees = factory.create_bool_set(True, 210, 211, 222)
        ConveyorBeltCollision = factory.create_bool_set(False, 624, 85, 629, 195, 1, 147, 184, 537, 204, 16, 59, 71, 535, 225, 676, 303, 335, 336, 333, 334, 667, 141, 81, 121, 183, 138, 244, 304, 105, 123, 685, 686, 687, 106, 354, 376, 579, 589, 37, 695, 696)
        SlimeCanContainItems = factory.create_bool_set(False, 1, 59, 147, 184, 537)
        FighterUsesDD2PortalAppearEffect = factory.create_bool_set(False, 552, 553, 554, 561, 562, 563, 555, 556, 557, 576, 577, 568, 569, 570, 571, 572, 573, 564, 565)
        NoEarlymodeLootWhenSpawnedFromStatue = factory.create_bool_set(False, 480, 82, 86, 170, 180, 171)
        NeedsExpertScaling = factory.create_bool_set(False, 25, 30, 665, 33, 112, 666, 261, 265, 371, 516, 519, 397, 396, 398, 491)
        ProjectileNPC = factory.create_bool_set(False, 25, 30, 665, 33, 112, 666, 261, 265, 371, 516, 519)
        SavesAndLoads = factory.create_bool_set(False, 422, 507, 517, 493)
        TrailCacheLength = factory.create_int_set(10, 402, 36, 519, 20, 522, 20, 620, 20, 677, 60, 685, 10)
        SearchSpawnSlotsInReverse = factory.create_bool_set(False, 222, 245)
        CannotSpawnInSlot0 = factory.create_bool_set(False, 134, 135, 136, 7, 8, 9, 10, 11, 12, 13, 14, 15, 39, 40, 41, 87, 88, 89, 90, 91, 92, 95, 96, 97, 98, 99, 100, 117, 118, 119, 375, 402, 412, 413, 414, 454, 455, 456, 457, 458, 459, 510, 511, 512, 513, 514, 515, 621, 622, 623)
        SkipUpdateInUnsyncedTiles = factory.create_bool_set(True, 135, 136, 14, 15, 8, 9, 11, 12, 40, 41, 96, 97, 99, 100, 118, 119, 622, 623, 511, 512, 514, 515, 455, 456, 457, 458, 459, 89, 90, 91, 92, 413, 414)
        UsesMultiplayerProximitySyncing = factory.create_bool_set(True, 398, 397, 396)
        NoMultiplayerSmoothingByType = factory.create_bool_set(False, 50, 657, 120, 245, 247, 248, 246, 370, 222, 398, 397, 396, 400, 401, 668, 70, 690)
        NoMultiplayerSmoothingByAI = factory.create_bool_set(False, 6, 8, 37)
        MPAllowedEnemies = factory.create_bool_set(False, 4, 13, 50, 126, 125, 134, 127, 128, 131, 129, 130, 222, 245, 266, 370, 657, 668)
        TownCritter = factory.create_bool_set(False, 46, 148, 149, 230, 299, 300, 303, 337, 361, 362, 364, 366, 367, 443, 445, 447, 538, 539, 540, 583, 584, 585, 592, 593, 602, 607, 608, 610, 616, 617, 625, 626, 627, 639, 640, 641, 642, 643, 644, 645, 646, 647, 648, 649, 650, 651, 652, 687, 688)
        CountsAsCritter = factory.create_bool_set(False, 46, 303, 337, 540, 443, 74, 297, 298, 442, 611, 689, 377, 446, 612, 613, 356, 444, 595, 596, 597, 598, 599, 600, 601, 604, 605, 357, 448, 374, 484, 355, 358, 606, 359, 360, 485, 486, 487, 148, 149, 55, 230, 592, 593, 299, 538, 539, 300, 447, 361, 445, 362, 363, 364, 365, 367, 366, 583, 584, 585, 602, 603, 607, 608, 609, 610, 616, 617, 625, 626, 627, 615, 639, 640, 641, 642, 643, 644, 645, 646, 647, 648, 649, 650, 651, 652, 653, 654, 655, 661, 669, 671, 672, 673, 674, 675, 677, 687, 688)
        HasNoPartyText = factory.create_bool_set(False, 441, 453)
        HatOffsetY = factory.create_int_set(0, 227, 4, 107, 2, 108, 2, 229, 4, 17, 2, 38, 8, 160, -10, 208, 2, 142, 2, 124, 2, 453, 2, 37, 4, 54, 4, 209, 4, 369, 6, 441, 6, 353, -2, 633, -2, 550, -2, 588, 2, 663, 2, 637, 0, 638, 0, 656, 4, 670, 0, 678, 0, 679, 0, 680, 0, 681, 0, 682, 0, 683, 0, 684, 0)
        FaceEmote = factory.create_int_set(0, 17, 101, 18, 102, 19, 103, 20, 104, 22, 105, 37, 106, 38, 107, 54, 108, 107, 109, 108, 110, 124, 111, 142, 112, 160, 113, 178, 114, 207, 115, 208, 116, 209, 117, 227, 118, 228, 119, 229, 120, 353, 121, 368, 122, 369, 123, 453, 124, 441, 125, 588, 140, 633, 141, 663, 145)
        DangerDetectRange = factory.create_int_set(-1, 38, 300, 17, 320, 107, 300, 19, 900, 22, 700, 124, 800, 228, 800, 178, 900, 18, 300, 229, 1000, 209, 1000, 54, 700, 108, 700, 160, 700, 20, 1200, 369, 300, 453, 300, 368, 900, 207, 60, 227, 800, 208, 400, 142, 500, 441, 50, 353, 60, 633, 100, 550, 120, 588, 120, 663, 700, 638, 250, 637, 250, 656, 250, 670, 250, 678, 250, 679, 250, 680, 250, 681, 250, 682, 250, 683, 250, 684, 250)
        ShimmerImmunity = factory.create_bool_set(False, 637, 638, 656, 670, 684, 678, 679, 680, 681, 682, 683, 356, 669, 676, 244, 677, 594, 667, 662, 5, 115, 116, 139, 245, 247, 248, 246, 249, 344, 325, 50, 535, 657, 658, 659, 660, 668, 25, 30, 33, 70, 72, 665, 666, 112, 516, 517, 518, 519, 520, 521, 522, 523, 381, 382, 383, 384, 385, 386, 387, 388, 389, 390, 391, 392, 393, 394, 395, 396, 397, 398, 399, 400, 401, 402, 403, 404, 405, 406, 407, 408, 409, 410, 411, 412, 413, 414, 415, 416, 417, 418, 419, 420, 421, 423, 424, 425, 426, 427, 428, 429, 548, 549, 551, 552, 553, 554, 555, 556, 557, 558, 559, 560, 561, 562, 563, 564, 565, 566, 567, 568, 569, 570, 571, 572, 573, 574, 575, 576, 577, 578)
        ShimmerTransformToItem = factory.create_int_set(-1, 651, 182, 644, 182, 650, 178, 643, 178, 649, 179, 642, 179, 648, 177, 641, 177, 640, 180, 647, 180, 646, 181, 639, 181, 652, 999, 645, 999, 448, 5341)
        ShimmerTownTransform = factory.create_bool_set(False, 22, 17, 18, 227, 207, 633, 588, 208, 369, 353, 38, 20, 550, 19, 107, 228, 54, 124, 441, 229, 160, 108, 178, 209, 142, 663, 37, 453, 368)
        ShimmerTransformToNPC = factory.create_int_set(-1, 3, 21, 132, 202, 186, 201, 187, 21, 188, 21, 189, 202, 200, 203, 590, 21, 1, 676, 302, 676, 335, 676, 336, 676, 334, 676, 333, 676, 225, 676, 141, 676, 16, 676, 147, 676, 184, 676, 537, 676, 204, 676, 81, 676, 183, 676, 138, 676, 121, 676, 591, 449, 430, 449, 436, 452, 432, 450, 433, 449, 434, 449, 435, 451, 614, 677, 74, 677, 297, 677, 298, 677, 673, 677, 672, 677, 671, 677, 675, 677, 674, 677, 362, 677, 363, 677, 364, 677, 365, 677, 608, 677, 609, 677, 602, 677, 603, 677, 611, 677, 689, 677, 148, 677, 149, 677, 46, 677, 303, 677, 337, 677, 299, 677, 538, 677, 55, 677, 607, 677, 615, 677, 625, 677, 626, 677, 688, 677, 361, 677, 687, 677, 484, 677, 604, 677, 358, 677, 355, 677, 616, 677, 617, 677, 654, 677, 653, 677, 655, 677, 585, 677, 584, 677, 583, 677, 595, 677, 596, 677, 600, 677, 597, 677, 598, 677, 599, 677, 357, 677, 377, 677, 606, 677, 359, 677, 360, 677, 367, 677, 366, 677, 300, 677, 610, 677, 612, 677, 487, 677, 486, 677, 485, 677, 669, 677, 356, 677, 661, 677, 374, 677, 442, 677, 443, 677, 444, 677, 601, 677, 445, 677, 592, 677, 446, 677, 605, 677, 447, 677, 627, 677, 539, 677, 613, 677)
        AttackTime = factory.create_int_set(-1, 38, 34, 17, 34, 107, 60, 19, 40, 22, 30, 124, 34, 228, 40, 178, 24, 18, 34, 229, 60, 209, 60, 54, 60, 108, 30, 160, 60, 20, 600, 369, 34, 453, 34, 368, 60, 207, 15, 227, 60, 208, 34, 142, 34, 441, 15, 353, 12, 633, 12, 550, 34, 588, 20, 663, 60, 638, -1, 637, -1, 656, -1, 670, -1, 678, -1, 679, -1, 680, -1, 681, -1, 682, -1, 683, -1, 684, -1)
        AttackAverageChance = factory.create_int_set(1, 38, 40, 17, 30, 107, 60, 19, 30, 22, 30, 124, 30, 228, 50, 178, 50, 18, 60, 229, 40, 209, 30, 54, 30, 108, 30, 160, 60, 20, 60, 369, 50, 453, 30, 368, 40, 207, 1, 227, 30, 208, 50, 142, 50, 441, 1, 353, 1, 633, 1, 550, 40, 588, 20, 663, 1, 638, 1, 637, 1, 656, 1, 670, 1, 678, 1, 679, 1, 680, 1, 681, 1, 682, 1, 683, 1, 684, 1)
        AttackType = factory.create_int_set(-1, 38, 0, 17, 0, 107, 0, 19, 1, 22, 1, 124, 0, 228, 1, 178, 1, 18, 0, 229, 1, 209, 1, 54, 2, 108, 2, 160, 2, 20, 2, 369, 0, 453, 0, 368, 1, 207, 3, 227, 1, 208, 0, 142, 0, 441, 3, 353, 3, 633, 0, 550, 0, 588, 0, 663, 2, 638, -1, 637, -1, 656, -1, 670, -1, 678, -1, 679, -1, 680, -1, 681, -1, 682, -1, 683, -1, 684, -1)
        PrettySafe = factory.create_int_set(-1, 19, 300, 22, 200, 124, 200, 228, 300, 178, 300, 229, 300, 209, 300, 54, 100, 108, 100, 160, 100, 20, 200, 368, 200, 227, 200)
        DemonEyes = factory.create_bool_set(False, 2, 190, 192, 193, 191, 194, 317, 318)
        Zombies = factory.create_bool_set(False, 3, 132, 186, 187, 188, 189, 200, 223, 161, 254, 255, 52, 53, 536, 319, 320, 321, 332, 436, 431, 432, 433, 434, 435, 331, 430, 590)
        Skeletons = factory.create_bool_set(False, 77, 449, 450, 451, 452, 481, 201, 202, 203, 21, 324, 110, 323, 293, 291, 322, 292, 197, 167, 44, 635)
        BossHeadTextures = factory.create_int_set(-1, 4, 0, 13, 2, 344, 3, 370, 4, 246, 5, 249, 5, 345, 6, 50, 7, 396, 8, 395, 9, 325, 10, 262, 11, 327, 13, 222, 14, 125, 15, 126, 20, 346, 17, 127, 18, 35, 19, 68, 19, 113, 22, 266, 23, 439, 24, 440, 24, 134, 25, 491, 26, 517, 27, 422, 28, 507, 29, 493, 30, 549, 35, 564, 32, 565, 32, 576, 33, 577, 33, 551, 34, 548, 36, 636, 37, 657, 38, 668, 39)
        PositiveNPCTypesExcludedFromDeathTally = factory.create_bool_set(False, 121, 384, 478, 479, 410, 472, 378)
        ShouldBeCountedAsBossForBestiary = factory.create_bool_set(False, 517, 422, 507, 493, 13, 664)
        ShouldBeCountedAsBossForRainbowBoulders = factory.create_bool_set(False, 517, 422, 507, 493, 13, 14, 15, 267, 36, 114, 664, 134, 135, 136, 128, 129, 130, 131, 245, 247, 248, 246, 491, 492, 392, 395, 394, 393, 564, 565, 576, 577, 551, 325, 327, 344, 345, 346)
        DangerThatPreventsOtherDangers = factory.create_bool_set(False, 517, 422, 507, 493, 399, 13, 14, 15)
        MustAlwaysDraw = factory.create_bool_set(False, 113, 114, 115, 116, 126, 125)
        NPCFramingGroup = factory.create_int_set(0, 18, 1, 20, 1, 208, 1, 178, 1, 124, 1, 353, 1, 633, 1, 369, 2, 160, 3, 637, 4, 638, 5, 656, 6, 670, 7, 678, 7, 679, 7, 680, 7, 681, 7, 682, 7, 683, 7, 684, 7)
        CanHitPastShimmer = factory.create_bool_set(False, 535, 5, 13, 14, 15, 666, 267, 36, 210, 211, 115, 116, 117, 118, 119, 658, 659, 660, 134, 135, 136, 139, 128, 131, 129, 130, 263, 264, 246, 249, 247, 248, 371, 372, 373, 566, 567, 440, 522, 523, 521, 454, 455, 456, 457, 458, 459, 397, 396, 400)
        TownNPCsFramingGroups = [[0, 0, 0, -2, -2, -2, 0, 0, 0, 0, -2, -2, -2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -2, -2, -2, 0, 0, 0, -2, -2, -2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -2, -2, -2, 0, 0, -2, -2, -2, -2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -2, 0, 0, 0, 0, -2, -2, -2, 0, 0, 0, 0, -2, -2, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 4, 6, 4, 2, 2, -2, -4, -6, -4, -2, -4, -4, -6, -6, -6, -4], [0, 0, 0, 0, 0, 0, 0, 0, -2, -2, -2, 0, 0, -2, -2, 0, 0, 4, 6, 6, 6, 6, 4, 4, 4, 4, 4, 4], [0, 0, -2, -4, -4, -2, 0, -2, 0, 0, 2, 4, 6, 4, 2, 0, -2, -4, -6, -6, -6, -6, -6, -6, -4, -2], [0, -2, 0, -2, -4, -6, -4, -2, 0, 0, 2, 2, 4, 2]]



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class SyncChestItem(SyncPacket):
    id = MessageID.SyncChestItem

    def __init__(
        self,
        chest_id: int = 0,
        item_slot: int = 0,
        stack: int = 0,
        prefix: int = 0,
        item_net_id: int = 0,
    ):
        self.chest_id = chest_id
        self.item_slot = item_slot
        self.stack = stack
        self.prefix = prefix
        self.item_net_id = item_net_id

    def read(self, reader: Reader):
        self.chest_id = reader.read_short()
        self.item_slot = reader.read_byte()
        self.stack = reader.read_short()
        self.prefix = reader.read_byte()
        self.item_net_id = reader.read_short()

    def write(self, writer: Writer):
        writer.write_short(self.chest_id)
        writer.write_byte(self.item_slot)
        writer.write_short(self.stack)
        writer.write_byte(self.prefix)
        writer.write_short(self.item_net_id)



"""
InvasionID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class InvasionID(IntEnum):
    NoneValue = 0
    CachedFrostMoon = 1
    GoblinArmy = 1
    CachedPumpkinMoon = 2
    SnowLegion = 2
    CachedInvasions = 3
    CachedOldOnesArmy = 3
    PirateInvasion = 3
    MartianMadness = 4




"""
GameEventClearedID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class GameEventClearedID(IntEnum):
    DefeatedGoblinArmy = 0
    DefeatedFrostArmy = 1
    DefeatedPirates = 2
    DefeatedMartians = 3
    DefeatedHalloweenTree = 4
    DefeatedHalloweenKing = 5
    DefeatedGolem = 6
    DefeatedFishron = 7
    DefeatedQueenBee = 8
    DefeatedAncientCultist = 9
    DefeatedMoonlord = 10
    DefeatedSlimeKing = 11
    DefeatedPlantera = 12
    DefeatedEyeOfCthulu = 13
    DefeatedEaterOfWorldsOrBrainOfChtulu = 14
    DefeatedSkeletron = 15
    DefeatedDestroyer = 16
    DefeatedTheTwins = 17
    DefeatedSkeletronPrime = 18
    DefeatedWallOfFleshAndStartedHardmode = 19
    DefeatedIceQueen = 20
    DefeatedChristmassTree = 21
    DefeatedSantank = 22
    DefeatedEmpressOfLight = 23
    DefeatedQueenSlime = 24
    DefeatedDeerclops = 25




from collections.abc import Callable
from typing import TypeVar

from terrex.event.context import EventContext
from terrex.event.types import (
    BaseEvent,
    ItemDroppedEvent,
    ItemDropUpdateEvent,
    ItemOwnerChangedEvent,
)

from .base import EventFilter

T = TypeVar("T", bound=BaseEvent)


class ItemEventFilter(EventFilter[T]):
    _event_type: type[T]

    def __init__(self, event_type: type[T], predicate: Callable[[T, EventContext], bool]):
        self._event_type = event_type
        self.predicate = predicate

    def matches(self, ctx: EventContext) -> T | None:
        event = ctx.event
        if not isinstance(event, self._event_type):
            return None
        if not self.predicate(event, ctx):
            return None
        return event


# Fabrics
def ItemDrop(item_id: int | None = None) -> ItemEventFilter[ItemDroppedEvent]:
    return ItemEventFilter(
        ItemDroppedEvent, lambda e, ctx: True if item_id is None else e.item.item_id == item_id
    )


def UpdateItemDrop(item_id: int | None = None) -> ItemEventFilter[ItemDropUpdateEvent]:
    return ItemEventFilter(
        ItemDropUpdateEvent, lambda e, ctx: True if item_id is None else e.item.item_id == item_id
    )


def UpdateItemOwner(item_id: int | None = None) -> ItemEventFilter[ItemOwnerChangedEvent]:
    return ItemEventFilter(
        ItemOwnerChangedEvent, lambda e, ctx: True if item_id is None else e.item_id == item_id
    )



"""
Ported from Terraria source: Terraria.WorldBuilding.Shapes.Tail

Thick line (tail) using Bresenham algorithm with pixel/tile thickness.
Equivalent to Terraria's Utils.PlotTileTale.
"""

import math
from dataclasses import dataclass

import numpy as np

from .base import GenAction, GenActionBulk, GenShape, Point


@dataclass
class Vector2D:
    """Local port of ReLogic.Utilities.Vector2D"""

    X: float
    Y: float

    def __add__(self, other: "Vector2D") -> "Vector2D":
        return Vector2D(self.X + other.X, self.Y + other.Y)

    def __mul__(self, scalar: float) -> "Vector2D":
        return Vector2D(self.X * scalar, self.Y * scalar)

    __rmul__ = __mul__  # commutative multiplication


class Tail(GenShape):
    """Thick line from origin to end_offset with given width (in tiles)."""

    def __init__(self, width: float, end_offset: Vector2D, quit_on_fail: bool = False):
        """
        width: line thickness in tiles
        end_offset: end position offset from origin (in tiles)
        """
        super().__init__(quit_on_fail)
        self._width = width  # in tiles
        self._endOffset = end_offset

    def perform(self, origin: Point, action: GenAction) -> bool:
        # Compute tile coordinates (offsets are in world units? but scaled to tiles)
        start_tile = origin
        end_tile_x = int(origin.x + self._endOffset.X)
        end_tile_y = int(origin.y + self._endOffset.Y)

        points = self._bresenham_thick(
            start_tile.x, start_tile.y, end_tile_x, end_tile_y, self._width
        )

        # Fast-path for vectorized actions
        if isinstance(action, GenActionBulk) and not self._quitOnFail:
            xs = np.array([p[0] for p in points])
            ys = np.array([p[1] for p in points])
            return action.apply_bulk(xs, ys)

        # Standard loop
        for x, y in points:
            if not self.unit_apply(action, origin, x, y) and self._quitOnFail:
                return False
        return True

    def _bresenham_thick(
        self, x0: int, y0: int, x1: int, y1: int, width: float
    ) -> list[tuple[int, int]]:
        """
        Bresenham line algorithm with thickness.
        width: thickness radius (full width ≈ 2*width + 1)
        Uses square approximation around each line point.
        """
        points = []
        dx = abs(x1 - x0)
        dy = abs(y1 - y0)
        sx = 1 if x0 < x1 else -1
        sy = 1 if y0 < y1 else -1
        err = dx - dy
        x, y = x0, y0

        half_w = int(math.floor(width / 2))

        while True:
            # Add thickness points (square around line point)
            for wx in range(-half_w, half_w + 1):
                for wy in range(-half_w, half_w + 1):
                    points.append((x + wx, y + wy))
            if x == x1 and y == y1:
                break
            e2 = 2 * err
            if e2 > -dy:
                err -= dy
                x += sx
            if e2 < dx:
                err += dx
                y += sy
        return points



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class TogglePvp(SyncPacket):
    id = MessageID.TogglePVP

    def __init__(self, player_id: int = 0, pvp_enabled: bool = False):
        self.player_id = player_id
        self.pvp_enabled = pvp_enabled

    def read(self, reader: Reader):
        self.player_id = reader.read_byte()
        self.pvp_enabled = reader.read_bool()

    def write(self, writer: Writer):
        writer.write_byte(self.player_id)
        writer.write_bool(self.pvp_enabled)



from terrex.id import MessageID
from terrex.net.streamer import Writer
from terrex.packet.base import ClientPacket


class FishOutNpc(ClientPacket):
    id = MessageID.FishOutNPC

    def __init__(self, x: int = 0, y: int = 0, npc_id: int = 0):
        self.x = x
        self.y = y
        self.npc_id = npc_id

    def write(self, writer: Writer):
        writer.write_short(self.x)
        writer.write_short(self.y)
        writer.write_short(self.npc_id)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ClientPacket


class ClientUUID(ClientPacket):
    id = MessageID.ClientUUID

    def __init__(self, uuid4: str = "01032c81-623f-4435-85e5-e0ec816b09ca"):
        self.uuid4 = uuid4

    def write(self, writer: Writer):
        writer.write_dotnet_string(self.uuid4)

    def read(self, reader: Reader) -> None:
        self.uuid4 = reader.read_dotnet_string()



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class SetCountsAsHostForGameplay(ServerPacket):
    id = MessageID.SetCountsAsHostForGameplay

    def __init__(self, player_id: int = 0, host: bool = False):
        self.player_id = player_id
        self.host = host

    def read(self, reader: Reader):
        self.player_id = reader.read_byte()
        self.host = reader.read_bool()



from dataclasses import dataclass

from terrex.net.creative_power.creative_power import CreativePower
from terrex.net.streamer import Reader, Writer


@dataclass()
class StartNoonImmediatelyPower(CreativePower):
    id: int = 2

    @classmethod
    def create(cls) -> "StartNoonImmediatelyPower":
        return cls()

    def read(self, reader: Reader) -> None:
        pass

    def write(self, writer: Writer) -> None:
        pass



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import ServerPacket


class SyncNPC(ServerPacket):
    id = MessageID.SyncNPC

    def __init__(
        self,
        npc_id: int = 0,
        pos: Vec2 | None = None,
        vel: Vec2 | None = None,
        target: int = 0,
        ai: list[float] | None = None,
        npc_net_id: int = 0,
        player_count_scale: int | None = None,
        strength_multiplier: float | None = None,
        life: int | None = None,
        release_owner: int | None = None,
        direction: int = -1,
        direction_y_positive: bool = False,
        sprite_direction: int = -1,
        full_life: bool = True,
        stats_scaled_gt1: bool = False,
        spawned_from_statue: bool = False,
        difficulty_flag: bool = False,
        spawn_needs_syncing: bool = False,
        shimmer_transparency_gt0: bool = False,
        life_size: int | None = None,
    ):
        self.npc_id = npc_id
        self.pos = pos or Vec2(0.0, 0.0)
        self.vel = vel or Vec2(0.0, 0.0)
        self.target = target
        self.ai = ai or [0.0] * 4
        self.npc_net_id = npc_net_id
        self.player_count_scale = player_count_scale
        self.strength_multiplier = strength_multiplier
        self.life = life
        self.release_owner = release_owner
        self.direction = direction
        self.direction_y_positive = direction_y_positive
        self.sprite_direction = sprite_direction
        self.full_life = full_life
        self.stats_scaled_gt1 = stats_scaled_gt1
        self.spawned_from_statue = spawned_from_statue
        self.difficulty_flag = difficulty_flag
        self.spawn_needs_syncing = spawn_needs_syncing
        self.shimmer_transparency_gt0 = shimmer_transparency_gt0
        self.life_size = life_size

    def read(self, reader: Reader):
        self.npc_id = reader.read_short()
        self.pos = Vec2.read(reader)
        self.vel = Vec2.read(reader)
        self.target = reader.read_ushort()
        bits1 = reader.read_byte()
        self.direction = 1 if (bits1 & 0x01) else -1
        self.direction_y_positive = bool(bits1 & 0x02)
        ai_active = [(bits1 & (0x04 << i)) != 0 for i in range(4)]
        self.sprite_direction = 1 if (bits1 & 0x40) else -1
        self.full_life = bool(bits1 & 0x80)
        bits2 = reader.read_byte()
        self.stats_scaled_gt1 = bool(bits2 & 0x01)
        self.spawned_from_statue = bool(bits2 & 0x02)
        self.difficulty_flag = bool(bits2 & 0x04)
        self.spawn_needs_syncing = bool(bits2 & 0x08)
        self.shimmer_transparency_gt0 = bool(bits2 & 0x10)
        self.ai = [0.0] * 4
        for i in range(4):
            if ai_active[i]:
                self.ai[i] = reader.read_float()
        self.npc_net_id = reader.read_short()
        self.player_count_scale = None
        if self.stats_scaled_gt1:
            self.player_count_scale = reader.read_byte()
        self.strength_multiplier = None
        if self.difficulty_flag:
            self.strength_multiplier = reader.read_float()
        self.life = None
        self.life_size = None
        if not self.full_life:
            self.life_size = reader.read_byte()
            size = self.life_size
            if size == 1:
                self.life = reader.read_sbyte()
            elif size == 2:
                self.life = reader.read_short()
            elif size == 4:
                self.life = reader.read_int()
        self.release_owner = None
        if not reader.eof():
            self.release_owner = reader.read_byte()



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.packet.base import ServerPacket


class Unused83(ServerPacket):
    # SetNPCKillCount
    id = MessageID.Unused83

    def __init__(self, npc_type: int = 0, kill_count: int = 0):
        self.npc_type = npc_type
        self.kill_count = kill_count

    def read(self, reader: Reader):
        self.npc_type = reader.read_short()
        self.kill_count = reader.read_int()



"""
PrefixID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class PrefixID(IntEnum):
    Large = 1
    Massive = 2
    Dangerous = 3
    Savage = 4
    Sharp = 5
    Pointy = 6
    Tiny = 7
    Terrible = 8
    Small = 9
    Dull = 10
    Unhappy = 11
    Bulky = 12
    Shameful = 13
    Heavy = 14
    Light = 15
    Sighted = 16
    Rapid = 17
    Hasty = 18
    Intimidating = 19
    Deadly = 20
    Staunch = 21
    Awful = 22
    Lethargic = 23
    Awkward = 24
    Powerful = 25
    Mystic = 26
    Adept = 27
    Masterful = 28
    Inept = 29
    Ignorant = 30
    Deranged = 31
    Intense = 32
    Taboo = 33
    Celestial = 34
    Furious = 35
    Keen = 36
    Superior = 37
    Forceful = 38
    Broken = 39
    Damaged = 40
    Shoddy = 41
    Quick = 42
    Deadly2 = 43
    Agile = 44
    Nimble = 45
    Murderous = 46
    Slow = 47
    Sluggish = 48
    Lazy = 49
    Annoying = 50
    Nasty = 51
    Manic = 52
    Hurtful = 53
    Strong = 54
    Unpleasant = 55
    Weak = 56
    Ruthless = 57
    Frenzying = 58
    Godly = 59
    Demonic = 60
    Zealous = 61
    Hard = 62
    Guarding = 63
    Armored = 64
    Warding = 65
    Arcane = 66
    Precise = 67
    Lucky = 68
    Jagged = 69
    Spiked = 70
    Angry = 71
    Menacing = 72
    Brisk = 73
    Fleeting = 74
    Hasty2 = 75
    Quick2 = 76
    Wild = 77
    Rash = 78
    Intrepid = 79
    Violent = 80
    Legendary = 81
    Unreal = 82
    Mythical = 83
    Legendary2 = 84
    Fabled = 85
    Loyal = 86
    Worthy = 87
    Focused = 88
    Patient = 89
    Rabid = 90
    IllTempered = 91
    Petty = 92
    Feeble = 93
    Skittish = 94
    Eager = 95
    Ballistic = 96
    Scraggling = 97
    Count = 98


class PrefixIDSets:
    class Main:
        factory = SetFactory(PrefixID.Count)
        ReducedNaturalChance = factory.create_bool_set(False, 7, 8, 9, 10, 11, 22, 23, 24, 29, 30, 31, 39, 40, 56, 41, 47, 48, 49)



from enum import IntEnum


class TeleportType(IntEnum):
    TeleporterTile = 0
    RodOfDiscord = 1
    TeleportationPotion = 2
    RecallPotion = 3
    Portal = 4
    MagicConch = 5
    DebugTeleport = 6
    DemonConch = 7
    PotionOfReturn = 8
    TeleportationPylon = 9
    QueenSlimeHook = 10
    ShellphoneSpawn = 11
    ShimmerTownNPCTransform = 12
    MysticFrog = 13
    NoEffect = 14



import re

from terrex.event.context import EventContext
from terrex.event.types import ChatEvent

from .base import EventFilter


class NewMessage(EventFilter[ChatEvent]):
    _event_type: type[ChatEvent] = ChatEvent

    def __init__(self, pattern: str | None = None):
        self.pattern = re.compile(pattern) if pattern else None

    def matches(self, ctx: EventContext) -> ChatEvent | None:
        event = ctx.event
        if not isinstance(event, ChatEvent):
            return None

        if self.pattern:
            match = self.pattern.search(event.text)
            if not match:
                return None
            event.pattern_match = match

        return event



from __future__ import annotations


class TileStack:
    def __init__(self) -> None:
        from terrex.net.structure.tile import Tile

        self._tiles: dict[tuple[int, int], Tile] = {}

    def get(self, x: int, y: int):
        return self._tiles.get((x, y), None)

    def set(self, x: int, y: int, tile) -> None:
        from terrex.net.structure.tile import Tile

        if not isinstance(tile, Tile):
            raise TypeError("tile must be a Tile instance")

        self._tiles[(x, y)] = tile

    def values(self):
        return self._tiles.values()

    def items(self):
        return self._tiles.items()

    def keys(self):
        return self._tiles.keys()

    def __len__(self) -> int:
        return len(self._tiles)

    def __iter__(self):
        return iter(self._tiles.values())



from dataclasses import dataclass

from terrex.localization.network_text import NetworkText
from terrex.net.enum.chat_command import ChatCommand
from terrex.net.enum.mode import NetMode
from terrex.net.streamer import Reader, Writer
from terrex.net.structure.rgb import Rgb

from .net_module import NetSyncModule


@dataclass()
class NetTextModule(NetSyncModule):
    id: int = 1
    chat_command_id: ChatCommand | None = None
    author_id: int | None = None
    text: str | None = None
    net_text: NetworkText | None = None
    color: Rgb | None = None

    @classmethod
    def create(
        cls,
        chat_command_id: ChatCommand,
        author_id: int | None = None,
        text: str | None = None,
        net_text: NetworkText | None = None,
        color: Rgb | None = None,
    ) -> "NetTextModule":
        obj = cls()
        obj.chat_command_id = chat_command_id
        obj.author_id = author_id
        obj.text = text
        obj.net_text = net_text
        obj.color = color
        return obj

    def read(self, reader: Reader) -> None:
        if reader.net_mode == NetMode.CLIENT:
            self.chat_command_id = ChatCommand(reader.read_dotnet_string())
            self.text = reader.read_dotnet_string()
        else:
            self.author_id = reader.read_byte()
            self.net_text = NetworkText.read(reader)
            self.color = Rgb.read(reader)

    def write(self, writer: Writer) -> None:
        if writer.net_mode == NetMode.CLIENT:
            if self.chat_command_id is None or self.text is None:
                raise ValueError("chat_command_id and text_str must not be None in CLIENT mode")
            writer.write_dotnet_string(str(self.chat_command_id))
            writer.write_dotnet_string(self.text)
        else:
            if self.author_id is None or self.net_text is None or self.color is None:
                raise ValueError("author_id, net_text, color must not be None in SERVER mode")
            writer.write_byte(self.author_id)
            self.net_text.write(writer)
            self.color.write(writer)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


class HitSwitch(SyncPacket):
    id = MessageID.HitSwitch

    def read(self, reader: Reader) -> None:
        self.x = reader.read_short()
        self.y = reader.read_short()

    def write(self, writer: Writer) -> None:
        writer.write_short(self.x)
        writer.write_short(self.y)



from enum import IntFlag

from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ClientPacket


class ToolMode(IntFlag):
    RED = 0x01
    GREEN = 0x02
    BLUE = 0x04
    YELLOW = 0x08
    ACTUATOR = 0x10
    CUTTER = 0x20

    @classmethod
    def read(cls, reader: Reader) -> 'ToolMode':
        return cls(reader.read_byte())

    def write(self, writer: Writer):
        writer.write_byte(self.value)


class MassWireOperation(ClientPacket):
    id = MessageID.MassWireOperation

    def __init__(
        self,
        start_x: int = 0,
        start_y: int = 0,
        end_x: int = 0,
        end_y: int = 0,
        tool_mode: ToolMode = ToolMode.RED,
    ):
        self.start_x = start_x
        self.start_y = start_y
        self.end_x = end_x
        self.end_y = end_y
        self.tool_mode = tool_mode

    def read(self, reader: Reader):
        self.start_x = reader.read_short()
        self.start_y = reader.read_short()
        self.end_x = reader.read_short()
        self.end_y = reader.read_short()
        self.tool_mode = ToolMode.read(reader)

    def write(self, writer: Writer):
        writer.write_short(self.start_x)
        writer.write_short(self.start_y)
        writer.write_short(self.end_x)
        writer.write_short(self.end_y)
        self.tool_mode.write(writer)



from dataclasses import dataclass

from terrex.net.streamer import Reader, Writer
from terrex.net.structure.liquid import Liquid

from .net_module import NetServerModule


@dataclass()
class NetLiquidModule(NetServerModule):
    id: int = 0
    liquids: list[Liquid] | None = None

    @classmethod
    def create(cls, liquids: list[Liquid]) -> "NetLiquidModule":
        obj = cls()
        obj.liquids = liquids
        return obj

    def read(self, reader: Reader) -> None:
        count = reader.read_ushort()
        self.liquids = [Liquid.read(reader) for _ in range(count)]

    def write(self, writer: Writer) -> None:
        if self.liquids is None:
            raise ValueError("liquids must not be None")
        writer.write_ushort(len(self.liquids))
        for liquid in self.liquids:
            liquid.write(writer)



from terrex.packet.base import Packet
from terrex.id import MessageID


class TShockPlaceholder(Packet):
    # Placeholder for TShock
    id = MessageID.TShockPlaceholder

    def read(self, reader):
        pass

    def write(self, writer):
        pass



"""
ProjectileID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class ProjectileID(IntEnum):
    NoneValue = 0
    WoodenArrowFriendly = 1
    FireArrow = 2
    Shuriken = 3
    UnholyArrow = 4
    JestersArrow = 5
    EnchantedBoomerang = 6
    VilethornBase = 7
    VilethornTip = 8
    Starfury = 9
    PurificationPowder = 10
    VilePowder = 11
    FallingStar = 12
    Hook = 13
    Bullet = 14
    BallofFire = 15
    MagicMissile = 16
    DirtBall = 17
    ShadowOrb = 18
    Flamarang = 19
    GreenLaser = 20
    Bone = 21
    WaterStream = 22
    Harpoon = 23
    SpikyBall = 24
    BallOHurt = 25
    BlueMoon = 26
    WaterBolt = 27
    Bomb = 28
    Dynamite = 29
    Grenade = 30
    SandBallFalling = 31
    IvyWhip = 32
    ThornChakram = 33
    Flamelash = 34
    Sunfury = 35
    MeteorShot = 36
    StickyBomb = 37
    HarpyFeather = 38
    MudBall = 39
    AshBallFalling = 40
    HellfireArrow = 41
    SandBallGun = 42
    Tombstone = 43
    DemonSickle = 44
    DemonScythe = 45
    DarkLance = 46
    Trident = 47
    ThrowingKnife = 48
    Spear = 49
    Glowstick = 50
    Seed = 51
    WoodenBoomerang = 52
    StickyGlowstick = 53
    PoisonedKnife = 54
    Stinger = 55
    EbonsandBallFalling = 56
    CobaltChainsaw = 57
    MythrilChainsaw = 58
    CobaltDrill = 59
    MythrilDrill = 60
    AdamantiteChainsaw = 61
    AdamantiteDrill = 62
    TheDaoofPow = 63
    MythrilHalberd = 64
    EbonsandBallGun = 65
    AdamantiteGlaive = 66
    PearlSandBallFalling = 67
    PearlSandBallGun = 68
    HolyWater = 69
    UnholyWater = 70
    SiltBall = 71
    BlueFairy = 72
    DualHookBlue = 73
    DualHookRed = 74
    HappyBomb = 75
    QuarterNote = 76
    EighthNote = 77
    TiedEighthNote = 78
    RainbowRodBullet = 79
    IceBlock = 80
    WoodenArrowHostile = 81
    FlamingArrow = 82
    EyeLaser = 83
    PinkLaser = 84
    Flames = 85
    PinkFairy = 86
    GreenFairy = 87
    PurpleLaser = 88
    CrystalBullet = 89
    CrystalShard = 90
    HolyArrow = 91
    HallowStar = 92
    MagicDagger = 93
    CrystalStorm = 94
    CursedFlameFriendly = 95
    CursedFlameHostile = 96
    CobaltNaginata = 97
    PoisonDart = 98
    Boulder = 99
    DeathLaser = 100
    EyeFire = 101
    BombSkeletronPrime = 102
    CursedArrow = 103
    CursedBullet = 104
    Gungnir = 105
    LightDisc = 106
    Hamdrax = 107
    Explosives = 108
    SnowBallHostile = 109
    BulletSnowman = 110
    Bunny = 111
    Penguin = 112
    IceBoomerang = 113
    UnholyTridentFriendly = 114
    UnholyTridentHostile = 115
    SwordBeam = 116
    BoneArrow = 117
    IceBolt = 118
    FrostBoltSword = 119
    FrostArrow = 120
    AmethystBolt = 121
    TopazBolt = 122
    SapphireBolt = 123
    EmeraldBolt = 124
    RubyBolt = 125
    DiamondBolt = 126
    Turtle = 127
    FrostBlastHostile = 128
    RuneBlast = 129
    MushroomSpear = 130
    Mushroom = 131
    TerraBeam = 132
    GrenadeI = 133
    RocketI = 134
    ProximityMineI = 135
    GrenadeII = 136
    RocketII = 137
    ProximityMineII = 138
    GrenadeIII = 139
    RocketIII = 140
    ProximityMineIII = 141
    GrenadeIV = 142
    RocketIV = 143
    ProximityMineIV = 144
    PureSpray = 145
    HallowSpray = 146
    CorruptSpray = 147
    MushroomSpray = 148
    CrimsonSpray = 149
    NettleBurstRight = 150
    NettleBurstLeft = 151
    NettleBurstEnd = 152
    TheRottedFork = 153
    TheMeatball = 154
    BeachBall = 155
    LightBeam = 156
    NightBeam = 157
    CopperCoin = 158
    SilverCoin = 159
    GoldCoin = 160
    PlatinumCoin = 161
    CannonballFriendly = 162
    Flare = 163
    Landmine = 164
    WebSlingerHook = 165
    SnowBallFriendly = 166
    RocketFireworkRed = 167
    RocketFireworkGreen = 168
    RocketFireworkBlue = 169
    RocketFireworkYellow = 170
    RopeCoil = 171
    FrostburnArrow = 172
    EnchantedBeam = 173
    IceSpike = 174
    BabyEater = 175
    JungleSpike = 176
    IcewaterSpit = 177
    ConfettiGun = 178
    SlushBall = 179
    BulletDeadeye = 180
    Bee = 181
    PossessedHatchet = 182
    Beenade = 183
    PoisonDartTrap = 184
    SpikyBallTrap = 185
    SpearTrap = 186
    FlamethrowerTrap = 187
    FlamesTrap = 188
    Wasp = 189
    MechanicalPiranha = 190
    Pygmy = 191
    Pygmy2 = 192
    Pygmy3 = 193
    Pygmy4 = 194
    PygmySpear = 195
    SmokeBomb = 196
    BabySkeletronHead = 197
    BabyHornet = 198
    TikiSpirit = 199
    PetLizard = 200
    GraveMarker = 201
    CrossGraveMarker = 202
    Headstone = 203
    Gravestone = 204
    Obelisk = 205
    Leaf = 206
    ChlorophyteBullet = 207
    Parrot = 208
    Truffle = 209
    Sapling = 210
    Wisp = 211
    PalladiumPike = 212
    PalladiumDrill = 213
    PalladiumChainsaw = 214
    OrichalcumHalberd = 215
    OrichalcumDrill = 216
    OrichalcumChainsaw = 217
    TitaniumTrident = 218
    TitaniumDrill = 219
    TitaniumChainsaw = 220
    FlowerPetal = 221
    ChlorophytePartisan = 222
    ChlorophyteDrill = 223
    ChlorophyteChainsaw = 224
    ChlorophyteArrow = 225
    CrystalLeaf = 226
    CrystalLeafShot = 227
    SporeCloud = 228
    ChlorophyteOrb = 229
    GemHookAmethyst = 230
    GemHookTopaz = 231
    GemHookSapphire = 232
    GemHookEmerald = 233
    GemHookRuby = 234
    GemHookDiamond = 235
    BabyDino = 236
    RainCloudMoving = 237
    RainCloudRaining = 238
    RainFriendly = 239
    CannonballHostile = 240
    CrimsandBallFalling = 241
    BulletHighVelocity = 242
    BloodCloudMoving = 243
    BloodCloudRaining = 244
    BloodRain = 245
    Stynger = 246
    FlowerPow = 247
    FlowerPowPetal = 248
    StyngerShrapnel = 249
    RainbowFront = 250
    RainbowBack = 251
    ChlorophyteJackhammer = 252
    BallofFrost = 253
    MagnetSphereBall = 254
    MagnetSphereBolt = 255
    SkeletronHand = 256
    FrostBeam = 257
    Fireball = 258
    EyeBeam = 259
    HeatRay = 260
    BoulderStaffOfEarth = 261
    GolemFist = 262
    IceSickle = 263
    RainNimbus = 264
    PoisonFang = 265
    BabySlime = 266
    PoisonDartBlowgun = 267
    EyeSpring = 268
    BabySnowman = 269
    Skull = 270
    BoxingGlove = 271
    Bananarang = 272
    ChainKnife = 273
    DeathSickle = 274
    SeedPlantera = 275
    PoisonSeedPlantera = 276
    ThornBall = 277
    IchorArrow = 278
    IchorBullet = 279
    GoldenShowerFriendly = 280
    ExplosiveBunny = 281
    VenomArrow = 282
    VenomBullet = 283
    PartyBullet = 284
    NanoBullet = 285
    ExplosiveBullet = 286
    GoldenBullet = 287
    GoldenShowerHostile = 288
    ConfettiMelee = 289
    ShadowBeamHostile = 290
    InfernoHostileBolt = 291
    InfernoHostileBlast = 292
    LostSoulHostile = 293
    ShadowBeamFriendly = 294
    InfernoFriendlyBolt = 295
    InfernoFriendlyBlast = 296
    LostSoulFriendly = 297
    SpiritHeal = 298
    Shadowflames = 299
    PaladinsHammerHostile = 300
    PaladinsHammerFriendly = 301
    SniperBullet = 302
    RocketSkeleton = 303
    VampireKnife = 304
    VampireHeal = 305
    EatersBite = 306
    TinyEater = 307
    FrostHydra = 308
    FrostBlastFriendly = 309
    BlueFlare = 310
    CandyCorn = 311
    JackOLantern = 312
    Spider = 313
    Squashling = 314
    BatHook = 315
    Bat = 316
    Raven = 317
    RottenEgg = 318
    BlackCat = 319
    BloodyMachete = 320
    FlamingJack = 321
    WoodHook = 322
    Stake = 323
    CursedSapling = 324
    FlamingWood = 325
    GreekFire1 = 326
    GreekFire2 = 327
    GreekFire3 = 328
    FlamingScythe = 329
    StarAnise = 330
    CandyCaneHook = 331
    ChristmasHook = 332
    FruitcakeChakram = 333
    Puppy = 334
    OrnamentFriendly = 335
    PineNeedleFriendly = 336
    Blizzard = 337
    RocketSnowmanI = 338
    RocketSnowmanII = 339
    RocketSnowmanIII = 340
    RocketSnowmanIV = 341
    NorthPoleWeapon = 342
    NorthPoleSpear = 343
    NorthPoleSnowflake = 344
    PineNeedleHostile = 345
    OrnamentHostile = 346
    OrnamentHostileShrapnel = 347
    FrostWave = 348
    FrostShard = 349
    Missile = 350
    Present = 351
    Spike = 352
    BabyGrinch = 353
    CrimsandBallGun = 354
    VenomFang = 355
    SpectreWrath = 356
    PulseBolt = 357
    WaterGun = 358
    FrostBoltStaff = 359
    BobberWooden = 360
    BobberReinforced = 361
    BobberFiberglass = 362
    BobberFisherOfSouls = 363
    BobberGolden = 364
    BobberMechanics = 365
    BobbersittingDuck = 366
    ObsidianSwordfish = 367
    Swordfish = 368
    SawtoothShark = 369
    LovePotion = 370
    FoulPotion = 371
    FishHook = 372
    Hornet = 373
    HornetStinger = 374
    FlyingImp = 375
    ImpFireball = 376
    SpiderHiver = 377
    SpiderEgg = 378
    BabySpider = 379
    ZephyrFish = 380
    BobberFleshcatcher = 381
    BobberHotline = 382
    Anchor = 383
    Sharknado = 384
    SharknadoBolt = 385
    Cthulunado = 386
    Retanimini = 387
    Spazmamini = 388
    MiniRetinaLaser = 389
    VenomSpider = 390
    JumperSpider = 391
    DangerousSpider = 392
    OneEyedPirate = 393
    SoulscourgePirate = 394
    PirateCaptain = 395
    SlimeHook = 396
    StickyGrenade = 397
    MiniMinotaur = 398
    MolotovCocktail = 399
    MolotovFire = 400
    MolotovFire2 = 401
    MolotovFire3 = 402
    TrackHook = 403
    Flairon = 404
    FlaironBubble = 405
    SlimeGun = 406
    Tempest = 407
    MiniSharkron = 408
    Typhoon = 409
    Bubble = 410
    CopperCoinsFalling = 411
    SilverCoinsFalling = 412
    GoldCoinsFalling = 413
    PlatinumCoinsFalling = 414
    RocketFireworksBoxRed = 415
    RocketFireworksBoxGreen = 416
    RocketFireworksBoxBlue = 417
    RocketFireworksBoxYellow = 418
    FireworkFountainYellow = 419
    FireworkFountainRed = 420
    FireworkFountainBlue = 421
    FireworkFountainRainbow = 422
    UFOMinion = 423
    Meteor1 = 424
    Meteor2 = 425
    Meteor3 = 426
    VortexChainsaw = 427
    VortexDrill = 428
    NebulaChainsaw = 429
    NebulaDrill = 430
    SolarFlareChainsaw = 431
    SolarFlareDrill = 432
    UFOLaser = 433
    ScutlixLaserFriendly = 434
    MartianTurretBolt = 435
    BrainScramblerBolt = 436
    GigaZapperSpear = 437
    RayGunnerLaser = 438
    LaserMachinegun = 439
    LaserMachinegunLaser = 440
    ScutlixLaserCrosshair = 441
    ElectrosphereMissile = 442
    Electrosphere = 443
    Xenopopper = 444
    LaserDrill = 445
    AntiGravityHook = 446
    SaucerDeathray = 447
    SaucerMissile = 448
    SaucerLaser = 449
    SaucerScrap = 450
    InfluxWaver = 451
    PhantasmalEye = 452
    DrillMountCrosshair = 453
    PhantasmalSphere = 454
    PhantasmalDeathray = 455
    MoonLeech = 456
    PhasicWarpEjector = 457
    PhasicWarpDisc = 458
    ChargedBlasterOrb = 459
    ChargedBlasterCannon = 460
    ChargedBlasterLaser = 461
    PhantasmalBolt = 462
    ViciousPowder = 463
    CultistBossIceMist = 464
    CultistBossLightningOrb = 465
    CultistBossLightningOrbArc = 466
    CultistBossFireBall = 467
    CultistBossFireBallClone = 468
    BeeArrow = 469
    StickyDynamite = 470
    SkeletonBone = 471
    WebSpit = 472
    SpelunkerGlowstick = 473
    BoneArrowFromMerchant = 474
    VineRopeCoil = 475
    SoulDrain = 476
    CrystalDart = 477
    CursedDart = 478
    IchorDart = 479
    CursedDartFlame = 480
    ChainGuillotine = 481
    ClingerStaff = 482
    SeedlerNut = 483
    SeedlerThorn = 484
    Hellwing = 485
    TendonHook = 486
    ThornHook = 487
    IlluminantHook = 488
    WormHook = 489
    CultistRitual = 490
    FlyingKnife = 491
    MagicLantern = 492
    CrystalVileShardHead = 493
    CrystalVileShardShaft = 494
    ShadowFlameArrow = 495
    ShadowFlame = 496
    ShadowFlameKnife = 497
    Nail = 498
    BabyFaceMonster = 499
    CrimsonHeart = 500
    DrManFlyFlask = 501
    Meowmere = 502
    StarWrath = 503
    Spark = 504
    SilkRopeCoil = 505
    WebRopeCoil = 506
    JavelinFriendly = 507
    JavelinHostile = 508
    ButchersChainsaw = 509
    ToxicFlask = 510
    ToxicCloud = 511
    ToxicCloud2 = 512
    ToxicCloud3 = 513
    NailFriendly = 514
    BouncyGlowstick = 515
    BouncyBomb = 516
    BouncyGrenade = 517
    CoinPortal = 518
    BombFish = 519
    FrostDaggerfish = 520
    CrystalPulse = 521
    CrystalPulse2 = 522
    ToxicBubble = 523
    IchorSplash = 524
    FlyingPiggyBank = 525
    CultistBossParticle = 526
    RichGravestone1 = 527
    RichGravestone2 = 528
    RichGravestone3 = 529
    RichGravestone4 = 530
    RichGravestone5 = 531
    BoneGloveProj = 532
    DeadlySphere = 533
    Code1 = 534
    MedusaHead = 535
    MedusaHeadRay = 536
    StardustSoldierLaser = 537
    Twinkle = 538
    StardustJellyfishSmall = 539
    StardustTowerMark = 540
    WoodYoyo = 541
    CorruptYoyo = 542
    CrimsonYoyo = 543
    JungleYoyo = 544
    Cascade = 545
    Chik = 546
    Code2 = 547
    Rally = 548
    Yelets = 549
    RedsYoyo = 550
    ValkyrieYoyo = 551
    Amarok = 552
    HelFire = 553
    Kraken = 554
    TheEyeOfCthulhu = 555
    BlackCounterweight = 556
    BlueCounterweight = 557
    GreenCounterweight = 558
    PurpleCounterweight = 559
    RedCounterweight = 560
    YellowCounterweight = 561
    FormatC = 562
    Gradient = 563
    Valor = 564
    BrainOfConfusion = 565
    GiantBee = 566
    SporeTrap = 567
    SporeTrap2 = 568
    SporeGas = 569
    SporeGas2 = 570
    SporeGas3 = 571
    SalamanderSpit = 572
    NebulaBolt = 573
    NebulaEye = 574
    NebulaSphere = 575
    NebulaLaser = 576
    VortexLaser = 577
    VortexVortexLightning = 578
    VortexVortexPortal = 579
    VortexLightning = 580
    VortexAcid = 581
    MechanicWrench = 582
    NurseSyringeHurt = 583
    NurseSyringeHeal = 584
    ClothiersCurse = 585
    DryadsWardCircle = 586
    PainterPaintball = 587
    PartyGirlGrenade = 588
    SantaBombs = 589
    TruffleSpore = 590
    MinecartMechLaser = 591
    MartianWalkerLaser = 592
    AncientDoomProjectile = 593
    BlowupSmoke = 594
    Arkhalis = 595
    DesertDjinnCurse = 596
    AmberBolt = 597
    BoneJavelin = 598
    BoneDagger = 599
    PortalGun = 600
    PortalGunBolt = 601
    PortalGunGate = 602
    Terrarian = 603
    TerrarianBeam = 604
    SpikedSlimeSpike = 605
    ScutlixLaser = 606
    SolarFlareRay = 607
    SolarCounter = 608
    StardustDrill = 609
    StardustChainsaw = 610
    SolarWhipSword = 611
    SolarWhipSwordExplosion = 612
    StardustCellMinion = 613
    StardustCellMinionShot = 614
    VortexBeater = 615
    VortexBeaterRocket = 616
    NebulaArcanum = 617
    NebulaArcanumSubshot = 618
    NebulaArcanumExplosionShot = 619
    NebulaArcanumExplosionShotShard = 620
    BloodWater = 621
    BlowupSmokeMoonlord = 622
    StardustGuardian = 623
    StardustGuardianExplosion = 624
    StardustDragon1 = 625
    StardustDragon2 = 626
    StardustDragon3 = 627
    StardustDragon4 = 628
    TowerDamageBolt = 629
    Phantasm = 630
    PhantasmArrow = 631
    LastPrismLaser = 632
    LastPrism = 633
    NebulaBlaze1 = 634
    NebulaBlaze2 = 635
    Daybreak = 636
    BouncyDynamite = 637
    MoonlordBullet = 638
    MoonlordArrow = 639
    MoonlordArrowTrail = 640
    MoonlordTurret = 641
    MoonlordTurretLaser = 642
    RainbowCrystal = 643
    RainbowCrystalExplosion = 644
    LunarFlare = 645
    LunarHookSolar = 646
    LunarHookVortex = 647
    LunarHookNebula = 648
    LunarHookStardust = 649
    SuspiciousTentacle = 650
    WireKite = 651
    StaticHook = 652
    CompanionCube = 653
    GeyserTrap = 654
    BeeHive = 655
    SandnadoFriendly = 656
    SandnadoHostile = 657
    SandnadoHostileMark = 658
    SpiritFlame = 659
    SkyFracture = 660
    BlackBolt = 661
    DD2JavelinHostile = 662
    DD2FlameBurstTowerT1 = 663
    DD2FlameBurstTowerT1Shot = 664
    DD2FlameBurstTowerT2 = 665
    DD2FlameBurstTowerT2Shot = 666
    DD2FlameBurstTowerT3 = 667
    DD2FlameBurstTowerT3Shot = 668
    Ale = 669
    DD2OgreStomp = 670
    DD2DrakinShot = 671
    DD2ElderWins = 672
    DD2DarkMageRaise = 673
    DD2DarkMageHeal = 674
    DD2DarkMageBolt = 675
    DD2OgreSpit = 676
    DD2BallistraTowerT1 = 677
    DD2BallistraTowerT2 = 678
    DD2BallistraTowerT3 = 679
    DD2BallistraProj = 680
    DD2GoblinBomb = 681
    DD2LightningBugZap = 682
    DD2OgreSmash = 683
    DD2SquireSonicBoom = 684
    DD2JavelinHostileT3 = 685
    DD2BetsyFireball = 686
    DD2BetsyFlameBreath = 687
    DD2LightningAuraT1 = 688
    DD2LightningAuraT2 = 689
    DD2LightningAuraT3 = 690
    DD2ExplosiveTrapT1 = 691
    DD2ExplosiveTrapT2 = 692
    DD2ExplosiveTrapT3 = 693
    DD2ExplosiveTrapT1Explosion = 694
    DD2ExplosiveTrapT2Explosion = 695
    DD2ExplosiveTrapT3Explosion = 696
    MonkStaffT1 = 697
    MonkStaffT1Explosion = 698
    MonkStaffT2 = 699
    MonkStaffT2Ghast = 700
    DD2PetDragon = 701
    DD2PetGhost = 702
    DD2PetGato = 703
    DD2ApprenticeStorm = 704
    DD2PhoenixBow = 705
    DD2PhoenixBowShot = 706
    MonkStaffT3 = 707
    MonkStaffT3_Alt = 708
    MonkStaffT3_AltShot = 709
    DD2BetsyArrow = 710
    ApprenticeStaffT3Shot = 711
    BookStaffShot = 712
    DD2Win = 713
    Celeb2Weapon = 714
    Celeb2Rocket = 715
    Celeb2RocketExplosive = 716
    Celeb2RocketLarge = 717
    Celeb2RocketExplosiveLarge = 718
    QueenBeeStinger = 719
    FallingStarSpawner = 720
    DirtGolfBall = 721
    GolfClubHelper = 722
    ManaCloakStar = 723
    BeeCloakStar = 724
    StarVeilStar = 725
    StarCloakStar = 726
    RollingCactus = 727
    SuperStar = 728
    SuperStarSlash = 729
    ThunderSpear = 730
    ThunderStaffShot = 731
    ThunderSpearShot = 732
    ToiletEffect = 733
    VoidLens = 734
    Terragrim = 735
    BlueDungeonDebris = 736
    GreenDungeonDebris = 737
    PinkDungeonDebris = 738
    GolfBallDyedBlack = 739
    GolfBallDyedBlue = 740
    GolfBallDyedBrown = 741
    GolfBallDyedCyan = 742
    GolfBallDyedGreen = 743
    GolfBallDyedLimeGreen = 744
    GolfBallDyedOrange = 745
    GolfBallDyedPink = 746
    GolfBallDyedPurple = 747
    GolfBallDyedRed = 748
    GolfBallDyedSkyBlue = 749
    GolfBallDyedTeal = 750
    GolfBallDyedViolet = 751
    GolfBallDyedYellow = 752
    AmberHook = 753
    MysticSnakeCoil = 754
    BatOfLight = 755
    SharpTears = 756
    DripplerFlail = 757
    VampireFrog = 758
    BabyBird = 759
    BobberBloody = 760
    PaperAirplaneA = 761
    PaperAirplaneB = 762
    RollingCactusSpike = 763
    UpbeatStar = 764
    SugarGlider = 765
    KiteBlue = 766
    KiteBlueAndYellow = 767
    KiteRed = 768
    KiteRedAndYellow = 769
    KiteYellow = 770
    KiteWyvern = 771
    Geode = 772
    ScarabBomb = 773
    SharkPup = 774
    BobberScarab = 775
    ClusterRocketI = 776
    ClusterGrenadeI = 777
    ClusterMineI = 778
    ClusterFragmentsI = 779
    ClusterRocketII = 780
    ClusterGrenadeII = 781
    ClusterMineII = 782
    ClusterFragmentsII = 783
    WetRocket = 784
    WetGrenade = 785
    WetMine = 786
    LavaRocket = 787
    LavaGrenade = 788
    LavaMine = 789
    HoneyRocket = 790
    HoneyGrenade = 791
    HoneyMine = 792
    MiniNukeRocketI = 793
    MiniNukeGrenadeI = 794
    MiniNukeMineI = 795
    MiniNukeRocketII = 796
    MiniNukeGrenadeII = 797
    MiniNukeMineII = 798
    DryRocket = 799
    DryGrenade = 800
    DryMine = 801
    GladiusStab = 802
    ClusterSnowmanRocketI = 803
    ClusterSnowmanRocketII = 804
    WetSnowmanRocket = 805
    LavaSnowmanRocket = 806
    HoneySnowmanRocket = 807
    MiniNukeSnowmanRocketI = 808
    MiniNukeSnowmanRocketII = 809
    DrySnowmanRocket = 810
    BloodShot = 811
    ShellPileFalling = 812
    BloodNautilusTears = 813
    BloodNautilusShot = 814
    LilHarpy = 815
    FennecFox = 816
    GlitteryButterfly = 817
    WhiteTigerPounce = 818
    BloodArrow = 819
    ChumBucket = 820
    BabyImp = 821
    KiteBoneSerpent = 822
    KiteWorldFeeder = 823
    KiteBunny = 824
    BabyRedPanda = 825
    KitePigron = 826
    KiteManEater = 827
    KiteJellyfishBlue = 828
    KiteJellyfishPink = 829
    KiteShark = 830
    StormTigerGem = 831
    StormTigerAttack = 832
    StormTigerTier1 = 833
    StormTigerTier2 = 834
    StormTigerTier3 = 835
    DandelionSeed = 836
    BookOfSkullsSkull = 837
    KiteSandShark = 838
    KiteBunnyCorrupt = 839
    KiteBunnyCrimson = 840
    BlandWhip = 841
    RulerStab = 842
    KiteGoldfish = 843
    KiteAngryTrapper = 844
    KiteKoi = 845
    KiteCrawltipede = 846
    SwordWhip = 847
    MaceWhip = 848
    ScytheWhip = 849
    KiteSpectrum = 850
    ReleaseDoves = 851
    KiteWanderingEye = 852
    KiteUnicorn = 853
    Plantero = 854
    ReleaseLantern = 855
    SparkleGuitar = 856
    FirstFractal = 857
    DynamiteKitten = 858
    BabyWerewolf = 859
    ShadowMimic = 860
    Football = 861
    ClusterSnowmanFragmentsI = 862
    ClusterSnowmanFragmentsII = 863
    Smolstar = 864
    SquirrelHook = 865
    BouncingShield = 866
    Shroomerang = 867
    TreeGlobe = 868
    WorldGlobe = 869
    FairyGlowstick = 870
    HallowBossSplitShotCore = 871
    HallowBossLastingRainbow = 872
    HallowBossRainbowStreak = 873
    HallowBossDeathAurora = 874
    VoltBunny = 875
    ZapinatorLaser = 876
    JoustingLance = 877
    ShadowJoustingLance = 878
    HallowJoustingLance = 879
    ZoologistStrikeGreen = 880
    KingSlimePet = 881
    EyeOfCthulhuPet = 882
    EaterOfWorldsPet = 883
    BrainOfCthulhuPet = 884
    SkeletronPet = 885
    QueenBeePet = 886
    DestroyerPet = 887
    TwinsPet = 888
    SkeletronPrimePet = 889
    PlanteraPet = 890
    GolemPet = 891
    DukeFishronPet = 892
    LunaticCultistPet = 893
    MoonLordPet = 894
    FairyQueenPet = 895
    PumpkingPet = 896
    EverscreamPet = 897
    IceQueenPet = 898
    MartianPet = 899
    DD2OgrePet = 900
    DD2BetsyPet = 901
    CombatWrench = 902
    WetBomb = 903
    LavaBomb = 904
    HoneyBomb = 905
    DryBomb = 906
    OrnamentStar = 907
    TitaniumStormShard = 908
    RockGolemRock = 909
    DirtBomb = 910
    DirtStickyBomb = 911
    CoolWhip = 912
    FireWhip = 913
    ThornWhip = 914
    RainbowWhip = 915
    ScytheWhipProj = 916
    CoolWhipProj = 917
    FireWhipProj = 918
    FairyQueenLance = 919
    QueenSlimeMinionBlueSpike = 920
    QueenSlimeMinionPinkBall = 921
    QueenSlimeSmash = 922
    FairyQueenSunDance = 923
    FairyQueenHymn = 924
    StardustPunch = 925
    QueenSlimeGelAttack = 926
    PiercingStarlight = 927
    DripplerFlailExtraBall = 928
    ZoologistStrikeRed = 929
    SantankMountRocket = 930
    FairyQueenMagicItemShot = 931
    FairyQueenRangedItemShot = 932
    FinalFractal = 933
    QueenSlimePet = 934
    QueenSlimeHook = 935
    GelBalloon = 936
    VolatileGelatinBall = 937
    CopperShortswordStab = 938
    TinShortswordStab = 939
    IronShortswordStab = 940
    LeadShortswordStab = 941
    SilverShortswordStab = 942
    TungstenShortswordStab = 943
    GoldShortswordStab = 944
    PlatinumShortswordStab = 945
    EmpressBlade = 946
    Mace = 947
    FlamingMace = 948
    TorchGod = 949
    PrincessWeapon = 950
    FlinxMinion = 951
    BoneWhip = 952
    DaybreakExplosion = 953
    WandOfSparkingSpark = 954
    StarCannonStar = 955
    BerniePet = 956
    GlommerPet = 957
    DeerclopsPet = 958
    PigPet = 959
    ChesterPet = 960
    DeerclopsIceSpike = 961
    DeerclopsRangedProjectile = 962
    AbigailMinion = 963
    InsanityShadowFriendly = 964
    InsanityShadowHostile = 965
    HoundiusShootius = 966
    HoundiusShootiusFireball = 967
    PewMaticHornShot = 968
    WeatherPainShot = 969
    AbigailCounter = 970
    TentacleSpike = 971
    NightsEdge = 972
    TrueNightsEdge = 973
    LightsBane = 974
    BloodButcherer = 975
    BladeOfGrass = 976
    Muramasa = 977
    Volcano = 978
    WandOfFrostingFrost = 979
    VenomDartTrap = 980
    SilverBullet = 981
    Excalibur = 982
    TrueExcalibur = 983
    TerraBlade2 = 984
    TerraBlade2Shot = 985
    FishingBobber = 986
    FishingBobberGlowingStar = 987
    FishingBobberGlowingLava = 988
    FishingBobberGlowingKrypton = 989
    FishingBobberGlowingXenon = 990
    FishingBobberGlowingArgon = 991
    FishingBobberGlowingViolet = 992
    FishingBobberGlowingRainbow = 993
    JunimoPet = 994
    JuminoStardropAnimation = 995
    MoonGlobe = 996
    TheHorsemansBlade = 997
    BlueChickenPet = 998
    HiveFive = 999
    Trimarang = 1000
    HorsemanPumpkin = 1001
    TNTBarrel = 1002
    Spiffo = 1003
    CavelingGardener = 1004
    MiniBoulder = 1005
    ShimmerArrow = 1006
    GasTrap = 1007
    SpelunkerFlare = 1008
    CursedFlare = 1009
    RainbowFlare = 1010
    ShimmerFlare = 1011
    Waffle = 1012
    BouncyBoulder = 1013
    LifeCrystalBoulder = 1014
    SandSpray = 1015
    SnowSpray = 1016
    DirtSpray = 1017
    DirtiestBlock = 1018
    Fertilizer = 1019
    JimsDrone = 1020
    MoonBoulder = 1021
    DeadCellsMushroomBoiMinion = 1022
    DeadCellsBarrel = 1023
    DeadCellsKillingDeckCard = 1024
    DeadCellsBarnacle = 1025
    DeadCellsBarnacleShot = 1026
    DeadCellsSwarmBiter = 1027
    CobWhip = 1028
    CorruptWhip = 1029
    CrimsonWhip = 1030
    MeteorWhip = 1031
    FlowerWhip = 1032
    EelWhip = 1033
    ConstellationWhip = 1034
    MoonLordWhip = 1035
    CobWhipSpider = 1036
    MeteorWhipMeteor = 1037
    FlowerWhipPetal = 1038
    ConstellationStar = 1039
    DeadCellsFlint = 1040
    DeadCellsFlintShot = 1041
    ShimmerGunStream = 1042
    DeadCellsFlintSlash = 1043
    DeadCellsMushroomBoiMinionExplosion = 1044
    MoonLordWhipProc = 1045
    Pufferfish = 1046
    RainbowBoulder = 1047
    Poulder = 1048
    MeteorStormMeteor = 1049
    AxeFairyPet = 1050
    SuperFertilizer = 1051
    Axearang = 1052
    LavaBoulder = 1053
    SpiderBoulder = 1054
    Ghoulder = 1055
    BoulderPet = 1056
    BoulderThatSpawnsPet = 1057
    FlaironFlail = 1058
    BluePhaseblade = 1059
    RedPhaseblade = 1060
    GreenPhaseblade = 1061
    PurplePhaseblade = 1062
    WhitePhaseblade = 1063
    YellowPhaseblade = 1064
    BluePhasesaber = 1065
    RedPhasesaber = 1066
    GreenPhasesaber = 1067
    PurplePhasesaber = 1068
    WhitePhasesaber = 1069
    YellowPhasesaber = 1070
    OrangePhaseblade = 1071
    OrangePhasesaber = 1072
    PoisonDartShotFromSlimes = 1073
    Keybrand = 1074
    PinkPhaseblade = 1075
    PinkPhasesaber = 1076
    FreezeBomb = 1077
    MeteorOre = 1078
    PinkCounterweight = 1079
    BirdDroppings = 1080
    MudBallPlayer = 1081
    AntlionClaw = 1082
    StylistKilLaKillScissorsIWish = 1083
    TorchGodHelper = 1084
    PrettyMirror = 1085
    SuperBomb = 1086
    SuperStickyBomb = 1087
    AcornSlingshotAcorn = 1088
    RainbowGlowstick = 1089
    RainbowBoulderPet = 1090
    StormLightning = 1091
    LibrarianSkeletonBook = 1092
    PalworldMinionCattiva = 1093
    PalworldMinionFoxsparks = 1094
    PalworldPetChillet = 1095
    PalworldPetChilletIgnis = 1096
    PalworldMinionFoxsparksFireball = 1097
    PalworldDigtoise = 1098
    SoundGun = 1099
    TrueCopperShortsword = 1100
    RainbowPhaseblade = 1101
    RainbowPhasesaber = 1102
    SlimeSpear = 1103
    SlimeWhip = 1104
    RemoteControlCar = 1105
    PalworldMinionFoxsparksFlames = 1106
    ObsidianFire = 1107
    ObsidianFire2 = 1108
    ObsidianFire3 = 1109
    SkyDragonsFuryElectrosphere = 1110
    Count = 1


class ProjectileIDSets:
    class Main:
        factory = SetFactory(ProjectileID.Count)
        DontApplyParryDamageBuff = factory.create_bool_set(False, 524, 321, 181, 566)
        BreaksFromToyBreaker = factory.create_bool_set(False, 1020, 1105, 861, 155, 855, 50, 53, 515, 473, 1089, 870, 721, 739, 740, 741, 742, 743, 744, 745, 746, 747, 748, 749, 750, 751, 752, 844, 766, 767, 822, 824, 839, 840, 846, 843, 828, 829, 845, 827, 826, 768, 769, 838, 830, 850, 853, 852, 823, 771, 770)
        IsAGravestone = factory.create_bool_set(False, 202, 201, 204, 43, 203, 205, 527, 528, 529, 530, 531)
        CopiesOwnerAttackCDToLocalImmunityOnSpawn = factory.create_bool_set(False, 1059, 1060, 1061, 1062, 1063, 1064, 1065, 1066, 1067, 1068, 1069, 1070, 1071, 1072, 1075, 1076, 1101, 1102, 1074)
        IsAPhaseblade = factory.create_bool_set(False, 1059, 1060, 1061, 1062, 1063, 1064, 1065, 1066, 1067, 1068, 1069, 1070, 1071, 1072, 1075, 1076, 1101, 1102, 1074)
        IsAWhip = factory.create_bool_set(False, 847, 841, 848, 849, 912, 913, 914, 915, 952, 1028, 1029, 1030, 1031, 1032, 1033, 1034, 1035, 1104)
        AllowsContactDamageFromJellyfish = factory.create_bool_set(False, 972, 982, 984, 997, 1043)
        ImmediatelyUpdatesNPCBuffFlags = factory.create_bool_set(False, 636)
        IsAnNPCAttachedExplosive = factory.create_bool_set(False, 636)
        RocketsSkipDamageForPlayers = factory.create_bool_set(False, 338, 339, 340, 341, 803, 804, 862, 863, 805, 806, 807, 808, 809, 810, 930, 1088)
        IsABombWithFuse = factory.create_bool_set(False, 28, 37, 516, 519, 910, 911, 1086, 1087, 906, 905, 904, 903, 773, 1077, 75, 102, 681, 470, 29, 637)
        IsAGolfBall = factory.create_bool_set(False, 721, 739, 740, 741, 742, 743, 744, 745, 746, 747, 748, 749, 750, 751, 752)
        IsDestroyedByWater = factory.create_bool_set(False, 85, 15, 188, 34, 1097, 1106)
        CanDistortWater = factory.create_bool_set(True, 7, 8, 152, 151, 150, 493, 494)
        MinionShot = factory.create_bool_set(False, 374, 376, 389, 195, 408, 433, 614, 818, 1044, 1097, 1106)
        SentryShot = factory.create_bool_set(False, 680, 664, 666, 668, 694, 695, 696, 644, 642, 378, 379, 309, 967, 1026)
        FallingBlockDoesNotFallThroughPlatforms = factory.create_bool_set(False, 71, 179, 40, 39, 17, 812, 411, 412, 413, 414, 31, 67, 56, 241, 42, 68, 65, 354)
        TrailingMode = factory.create_int_set(-1, 94, 0, 301, 0, 388, 0, 385, 0, 408, 0, 409, 0, 435, 0, 436, 0, 437, 0, 438, 0, 452, 0, 459, 0, 462, 0, 502, 0, 503, 0, 466, 1, 532, 0, 533, 0, 573, 0, 580, 1, 582, 0, 585, 0, 592, 0, 601, 0, 617, 0, 636, 0, 638, 0, 639, 0, 640, 0, 424, 0, 425, 0, 426, 0, 1037, 0, 660, 0, 661, 0, 671, 2, 664, 0, 666, 0, 668, 0, 675, 0, 680, 2, 682, 0, 684, 0, 686, 2, 700, 0, 706, 0, 709, 0, 710, 2, 711, 2, 712, 0, 715, 2, 716, 2, 717, 2, 718, 2, 261, 0, 721, 0, 729, 2, 732, 0, 731, 0, 739, 0, 740, 0, 741, 0, 742, 0, 743, 0, 744, 0, 745, 0, 746, 0, 747, 0, 748, 0, 749, 0, 750, 0, 751, 0, 752, 0, 755, 2, 766, 2, 767, 2, 768, 2, 769, 2, 770, 2, 771, 2, 811, 2, 814, 2, 822, 2, 823, 2, 824, 2, 826, 2, 827, 2, 828, 2, 829, 2, 830, 2, 838, 2, 839, 2, 840, 2, 843, 2, 844, 2, 845, 2, 846, 2, 850, 2, 852, 2, 853, 2, 856, 0, 857, 0, 864, 2, 873, 2, 872, 2, 833, 2, 834, 2, 835, 2, 818, 2, 902, 0, 883, 0, 887, 0, 893, 0, 894, 0, 909, 0, 916, 2, 34, 3, 16, 3, 79, 3, 931, 2, 933, 4, 946, 2, 964, 0, 965, 0, 977, 2, 976, 2, 973, 2, 106, 5, 85, 3, 1001, 3, 1020, 2, 1024, 2, 1039, 2, 1026, 0, 1045, 2, 1047, 0, 1055, 0, 1089, 0, 1090, 0, 1100, 4, 1097, 2, 1106, 3)
        TrailCacheLength = factory.create_int_set(10, 466, 20, 502, 25, 580, 20, 636, 20, 640, 20, 686, 20, 711, 20, 715, 20, 716, 20, 717, 20, 718, 20, 261, 20, 721, 20, 729, 20, 731, 20, 739, 20, 740, 20, 741, 20, 742, 20, 743, 20, 744, 20, 745, 20, 746, 20, 747, 20, 748, 20, 749, 20, 750, 20, 751, 20, 752, 20, 766, 60, 767, 60, 768, 60, 769, 60, 770, 60, 771, 80, 814, 40, 822, 80, 823, 80, 824, 60, 826, 60, 827, 65, 828, 60, 829, 60, 830, 80, 838, 80, 839, 60, 840, 60, 843, 60, 844, 65, 845, 80, 846, 80, 850, 80, 852, 60, 853, 60, 856, 2, 857, 2, 864, 60, 873, 60, 872, 120, 833, 20, 834, 20, 835, 20, 818, 20, 883, 41, 887, 51, 893, 71, 894, 10, 909, 5, 916, 20, 34, 30, 16, 30, 79, 60, 931, 20, 933, 60, 946, 20, 977, 30, 976, 60, 973, 20, 106, 15, 85, 60, 1001, 12, 1020, 20, 1024, 40, 1039, 20, 1026, 10, 1045, 30, 1047, 25, 1089, 25, 1090, 10, 1100, 60, 1097, 20, 1106, 60)
        LightPet = factory.create_bool_set(False, 18, 500, 72, 87, 86, 211, 492, 650, 702, 891, 896, 895)
        DontCancelChannelOnKill = factory.create_bool_set(False, 190)
        CultistIsResistantTo = factory.create_bool_set(False, 207, 182, 338, 339, 340, 341, 266, 390, 391, 392, 307, 316, 190, 227, 255, 297, 317, 321, 356, 407, 376, 374, 379, 408, 389, 388, 533, 405, 409, 535, 536, 484, 477, 16, 34, 79, 634, 635, 616, 189, 181, 566, 837, 659, 803, 804, 805, 806, 807, 808, 809, 810, 856, 931, 248, 393, 394, 395, 378, 379, 664, 666, 668, 680, 625, 626, 627, 628, 613, 614, 309, 195, 433, 755, 864, 758, 759, 832, 833, 834, 835, 951, 644, 642, 946, 930, 937, 225, 285, 917, 700, 916, 640, 221, 606, 567, 568, 569, 570, 571, 617, 618, 619, 620, 963, 967, 1022, 1026, 1036, 1038, 1037, 1039, 1093, 1097)
        IsADD2Turret = factory.create_bool_set(False, 663, 665, 667, 677, 678, 679, 688, 689, 690, 691, 692, 693)
        TurretFeature = factory.create_bool_set(False)
        MinionTargettingFeature = factory.create_bool_set(False, 191, 192, 193, 194, 266, 317, 373, 375, 387, 388, 390, 393, 407, 423, 533, 613, 625, 755, 758, 759, 831, 833, 834, 835, 864, 946, 951, 963, 970, 1022, 1093, 1094, 377, 308, 643, 641, 663, 665, 667, 677, 678, 679, 688, 689, 690, 691, 692, 693, 966, 1025)
        MinionSacrificable = factory.create_bool_set(False, 191, 192, 193, 194, 266, 317, 373, 375, 387, 388, 390, 393, 407, 423, 533, 613, 755, 758, 759, 831, 970, 864, 946, 951, 963, 1022, 1093, 1094, 625, 626, 627, 628)
        NeedsUUID = factory.create_bool_set(False, 625, 626, 627, 628)
        StardustDragon = factory.create_bool_set(False, 625, 626, 627, 628)
        StormTiger = factory.create_bool_set(False, 833, 834, 835)
        StormTigerIds = [833, 834, 835]
        IsARocketThatDealsDoubleDamageToPrimaryEnemy = factory.create_bool_set(False, 134, 137, 140, 143, 776, 780, 793, 796, 799, 784, 787, 790, 246)
        IsAMineThatDealsTripleDamageWhenStationary = factory.create_bool_set(False, 135, 138, 141, 144, 778, 782, 795, 798, 801, 786, 789, 792)
        NoLiquidDistortion = factory.create_bool_set(False, 511, 512, 513)
        DismountsPlayersOnHit = factory.create_bool_set(False, 877, 878, 879)
        NoMeleeSpeedVelocityScaling = factory.create_bool_set(False, 699, 707, 877, 878, 879, 985, 973)
        DrawScreenCheckFluff = factory.create_int_set(480, 461, 1600, 632, 1600, 447, 1600, 455, 2400, 754, 1600, 872, 1600, 873, 1600, 871, 1600, 919, 2400, 923, 2400, 931, 960, 16, 960, 34, 960, 79, 960, 933, 480, 642, 2400, 106, 960, 1020, 960, 1024, 960, 1039, 960, 1026, 960, 1100, 480, 23, 960, 262, 960, 271, 960, 273, 960, 481, 960)
        CanHitPastShimmer = factory.create_bool_set(False, 605, 270, 719, 961, 962, 926, 922, 100, 84, 83, 96, 101, 102, 275, 276, 277, 258, 259, 384, 385, 386, 874, 872, 873, 871, 683, 676, 670, 675, 686, 687, 467, 468, 464, 465, 466, 526, 456, 462, 455, 452, 454, 949, 1041)
        TrackMinionSpawnFromItemUse = factory.create_bool_set(True, 388, 625, 627, 628)



from terrex.id import MessageID
from terrex.net.streamer import Reader
from terrex.net.structure.rgb import Rgb
from terrex.net.structure.vec2 import Vec2
from terrex.packet.base import ServerPacket


class CombatTextInt(ServerPacket):
    id = MessageID.CombatTextInt

    def __init__(self, pos: Vec2 | None = None, color: Rgb | None = None, heal_amount: int = 0):
        self.pos = pos or Vec2(0.0, 0.0)
        self.color = color or Rgb(0, 0, 0)
        self.heal_amount = heal_amount

    def read(self, reader: Reader):
        self.pos = Vec2.read(reader)
        self.color = Rgb.read(reader)
        self.heal_amount = reader.read_int()



"""
BuffID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from terrex.id.set_factory import SetFactory
from enum import IntEnum, auto


class BuffID(IntEnum):
    ObsidianSkin = 1
    Regeneration = 2
    Swiftness = 3
    Gills = 4
    Ironskin = 5
    ManaRegeneration = 6
    MagicPower = 7
    Featherfall = 8
    Spelunker = 9
    Invisibility = 10
    Shine = 11
    NightOwl = 12
    Battle = 13
    Thorns = 14
    WaterWalking = 15
    Archery = 16
    Hunter = 17
    Gravitation = 18
    ShadowOrb = 19
    Poisoned = 20
    PotionSickness = 21
    Darkness = 22
    Cursed = 23
    OnFire = 24
    Tipsy = 25
    WellFed = 26
    FairyBlue = 27
    Werewolf = 28
    Clairvoyance = 29
    Bleeding = 30
    Confused = 31
    Slow = 32
    Weak = 33
    Merfolk = 34
    Silenced = 35
    BrokenArmor = 36
    Horrified = 37
    TheTongue = 38
    CursedInferno = 39
    PetBunny = 40
    BabyPenguin = 41
    PetTurtle = 42
    PaladinsShield = 43
    Frostburn = 44
    BabyEater = 45
    Chilled = 46
    Frozen = 47
    Honey = 48
    Pygmies = 49
    BabySkeletronHead = 50
    BabyHornet = 51
    TikiSpirit = 52
    PetLizard = 53
    PetParrot = 54
    BabyTruffle = 55
    PetSapling = 56
    Wisp = 57
    RapidHealing = 58
    ShadowDodge = 59
    LeafCrystal = 60
    BabyDinosaur = 61
    IceBarrier = 62
    Panic = 63
    BabySlime = 64
    EyeballSpring = 65
    BabySnowman = 66
    Burning = 67
    Suffocation = 68
    Ichor = 69
    Venom = 70
    WeaponImbueVenom = 71
    Midas = 72
    WeaponImbueCursedFlames = 73
    WeaponImbueFire = 74
    WeaponImbueGold = 75
    WeaponImbueIchor = 76
    WeaponImbueNanites = 77
    WeaponImbueConfetti = 78
    WeaponImbuePoison = 79
    Blackout = 80
    PetSpider = 81
    Squashling = 82
    Ravens = 83
    BlackCat = 84
    CursedSapling = 85
    WaterCandle = 86
    Campfire = 87
    ChaosState = 88
    HeartLamp = 89
    Rudolph = 90
    Puppy = 91
    BabyGrinch = 92
    AmmoBox = 93
    ManaSickness = 94
    BeetleEndurance1 = 95
    BeetleEndurance2 = 96
    BeetleEndurance3 = 97
    BeetleMight1 = 98
    BeetleMight2 = 99
    BeetleMight3 = 100
    FairyRed = 101
    FairyGreen = 102
    Wet = 103
    Mining = 104
    Heartreach = 105
    Calm = 106
    Builder = 107
    Titan = 108
    Flipper = 109
    Summoning = 110
    Dangersense = 111
    AmmoReservation = 112
    Lifeforce = 113
    Endurance = 114
    Rage = 115
    Inferno = 116
    Wrath = 117
    Minecart = 118
    Lovestruck = 119
    Stinky = 120
    Fishing = 121
    Sonar = 122
    Crate = 123
    Warmth = 124
    HornetMinion = 125
    ImpMinion = 126
    ZephyrFish = 127
    BunnyMount = 128
    PigronMount = 129
    SlimeMount = 130
    TurtleMount = 131
    BeeMount = 132
    SpiderMinion = 133
    TwinEyesMinion = 134
    PirateMinion = 135
    MiniMinotaur = 136
    Slimed = 137
    MinecartLegacyUnused = 138
    SharknadoMinion = 139
    UFOMinion = 140
    UFOMount = 141
    DrillMount = 142
    ScutlixMount = 143
    Electrified = 144
    MoonLeech = 145
    Sunflower = 146
    MonsterBanner = 147
    Rabies = 148
    Webbed = 149
    Bewitched = 150
    SoulDrain = 151
    MagicLantern = 152
    ShadowFlame = 153
    BabyFaceMonster = 154
    CrimsonHeart = 155
    Stoned = 156
    PeaceCandle = 157
    StarInBottle = 158
    Sharpened = 159
    Dazed = 160
    DeadlySphere = 161
    UnicornMount = 162
    Obstructed = 163
    VortexDebuff = 164
    DryadsWard = 165
    MinecartMech = 166
    MinecartMechLegacyUnused = 167
    CuteFishronMount = 168
    BoneJavelin = 169
    SolarShield1 = 170
    SolarShield2 = 171
    SolarShield3 = 172
    NebulaUpLife1 = 173
    NebulaUpLife2 = 174
    NebulaUpLife3 = 175
    NebulaUpMana1 = 176
    NebulaUpMana2 = 177
    NebulaUpMana3 = 178
    NebulaUpDmg1 = 179
    NebulaUpDmg2 = 180
    NebulaUpDmg3 = 181
    StardustMinion = 182
    StardustMinionBleed = 183
    MinecartWood = 184
    MinecartWoodLegacyUnused = 185
    DryadsWardDebuff = 186
    StardustGuardianMinion = 187
    StardustDragonMinion = 188
    Daybreak = 189
    SuspiciousTentacle = 190
    CompanionCube = 191
    SugarRush = 192
    BasiliskMount = 193
    WindPushed = 194
    WitheredArmor = 195
    WitheredWeapon = 196
    OgreSpit = 197
    ParryDamageBuff = 198
    NoBuilding = 199
    PetDD2Gato = 200
    PetDD2Ghost = 201
    PetDD2Dragon = 202
    BetsysCurse = 203
    Oiled = 204
    BallistaPanic = 205
    WellFed2 = 206
    WellFed3 = 207
    DesertMinecart = 208
    DesertMinecartLegacyUnused = 209
    FishMinecart = 210
    FishMinecartLegacyUnused = 211
    GolfCartMount = 212
    BatOfLight = 213
    VampireFrog = 214
    CatBast = 215
    BabyBird = 216
    UpbeatStar = 217
    SugarGlider = 218
    SharkPup = 219
    BeeMinecart = 220
    BeeMinecartLegacyUnused = 221
    LadybugMinecart = 222
    LadybugMinecartLegacyUnused = 223
    PigronMinecart = 224
    PigronMinecartLegacyUnused = 225
    SunflowerMinecart = 226
    SunflowerMinecartLegacyUnused = 227
    HellMinecart = 228
    HellMinecartLegacyUnused = 229
    WitchBroom = 230
    ShroomMinecart = 231
    ShroomMinecartLegacyUnused = 232
    AmethystMinecart = 233
    AmethystMinecartLegacyUnused = 234
    TopazMinecart = 235
    TopazMinecartLegacyUnused = 236
    SapphireMinecart = 237
    SapphireMinecartLegacyUnused = 238
    EmeraldMinecart = 239
    EmeraldMinecartLegacyUnused = 240
    RubyMinecart = 241
    RubyMinecartLegacyUnused = 242
    DiamondMinecart = 243
    DiamondMinecartLegacyUnused = 244
    AmberMinecart = 245
    AmberMinecartLegacyUnused = 246
    BeetleMinecart = 247
    BeetleMinecartLegacyUnused = 248
    MeowmereMinecart = 249
    MeowmereMinecartLegacyUnused = 250
    PartyMinecart = 251
    PartyMinecartLegacyUnused = 252
    PirateMinecart = 253
    PirateMinecartLegacyUnused = 254
    SteampunkMinecart = 255
    SteampunkMinecartLegacyUnused = 256
    Lucky = 257
    LilHarpy = 258
    FennecFox = 259
    GlitteryButterfly = 260
    BabyImp = 261
    BabyRedPanda = 262
    StormTiger = 263
    Plantero = 264
    Flamingo = 265
    DynamiteKitten = 266
    BabyWerewolf = 267
    ShadowMimic = 268
    CoffinMinecart = 269
    CoffinMinecartLegacyUnused = 270
    Smolstar = 271
    DiggingMoleMinecart = 272
    DiggingMoleMinecartLegacyUnused = 273
    VoltBunny = 274
    PaintedHorseMount = 275
    MajesticHorseMount = 276
    DarkHorseMount = 277
    PogoStickMount = 278
    PirateShipMount = 279
    SpookyWoodMount = 280
    SantankMount = 281
    WallOfFleshGoatMount = 282
    DarkMageBookMount = 283
    KingSlimePet = 284
    EyeOfCthulhuPet = 285
    EaterOfWorldsPet = 286
    BrainOfCthulhuPet = 287
    SkeletronPet = 288
    QueenBeePet = 289
    DestroyerPet = 290
    TwinsPet = 291
    SkeletronPrimePet = 292
    PlanteraPet = 293
    GolemPet = 294
    DukeFishronPet = 295
    LunaticCultistPet = 296
    MoonLordPet = 297
    FairyQueenPet = 298
    PumpkingPet = 299
    EverscreamPet = 300
    IceQueenPet = 301
    MartianPet = 302
    DD2OgrePet = 303
    DD2BetsyPet = 304
    LavaSharkMount = 305
    TitaniumStorm = 306
    # BlandWhipEnemyDebuff = 307
    SwordWhipPlayerBuff = 308
    # SwordWhipNPCDebuff = 309
    ScytheWhipEnemyDebuff = 310
    ScytheWhipPlayerBuff = 311
    CoolWhipPlayerBuff = 312
    # FlameWhipEnemyDebuff = 313
    ThornWhipPlayerBuff = 314
    # ThornWhipNPCDebuff = 315
    # RainbowWhipNPCDebuff = 316
    QueenSlimePet = 317
    QueenSlimeMount = 318
    # MaceWhipNPCDebuff = 319
    GelBalloonBuff = 320
    BrainOfConfusionBuff = 321
    EmpressBlade = 322
    OnFire3 = 323
    Frostburn2 = 324
    FlinxMinion = 325
    # BoneWhipNPCDebuff = 326
    BerniePet = 327
    GlommerPet = 328
    DeerclopsPet = 329
    PigPet = 330
    ChesterPet = 331
    NeutralHunger = 332
    Hunger = 333
    Starving = 334
    AbigailMinion = 335
    HeartyMeal = 336
    TentacleSpike = 337
    FartMinecart = 338
    FartMinecartLegacyUnused = 339
    # CoolWhipNPCDebuff = 340
    DualSlimePet = 341
    WolfMount = 342
    BiomeSight = 343
    BloodButcherer = 344
    JunimoPet = 345
    TerraFartMinecart = 346
    TerraFartMinecartLegacyUnused = 347
    WarTable = 348
    BlueChickenPet = 349
    ShadowCandle = 350
    Spiffo = 351
    CavelingGardener = 352
    Shimmer = 353
    DirtiestBlock = 354
    DeadCellsMushroomBoiMinion = 355
    DeadCellsSwarmBiter = 356
    # CobWhipNPCDebuff = 357
    # CorruptWhipNPCDebuff = 358
    # CrimsonWhipNPCDebuff = 359
    # MeteorWhipNPCDebuff = 360
    # FlowerWhipNPCDebuff = 361
    EelWhipNPCDebuff = 362
    # ConstellationWhipNPCDebuff = 363
    # MoonLordWhipNPCDebuff = 364
    CobWhipPlayerBuff = 365
    DeadCellsPotionStation = 366
    # FlowerWhipNPCDebuffProc = 367
    # MoonLordWhipNPCDebuffProc = 368
    # MeteorWhipNPCDebuffProc = 369
    VelociraptorMount = 370
    Pufferfish = 371
    AxeFairyPet = 372
    BoulderPet = 373
    RatMount = 374
    Hemorrhage = 375
    TorchGodPotion = 376
    BatMount = 377
    RollerSkatesMount = 378
    RollerSkatesGreenMount = 379
    RollerSkatesWhiteMount = 380
    RollerSkatesPinkMount = 381
    RainbowBoulderPet = 382
    Kite = 383
    PixieMount = 384
    PalworldMinionCattiva = 385
    PalworldMinionFoxsparks = 386
    PalworldPetChillet = 387
    PalworldPetChilletIgnis = 388
    Count = 389


class BuffIDSets:
    class Main:
        factory = SetFactory(BuffID.Count)
        IsWellFed = factory.create_bool_set(False, 26, 206, 207)
        IsFedState = factory.create_bool_set(False, 26, 206, 207, 332, 333, 334)
        SortingPriorityFoodBuffs = factory.create_int_set(-1, 207, 4, 206, 3, 26, 2, 25, 1)
        IsAnNPCWhipDebuff = factory.create_bool_set(False, 310, 362)
        TimeLeftDoesNotDecrease = factory.create_bool_set(False, 28, 334, 29, 159, 150, 93, 348, 366)
        CanBeRemovedByNetMessage = factory.create_bool_set(False)
        IsAFlaskBuff = factory.create_bool_set(False, 71, 72, 73, 74, 75, 76, 77, 78, 79)
        BuffTimeIsExtendedWithGameDifficulty = factory.create_bool_set(False, 20, 22, 23, 24, 323, 30, 31, 32, 33, 35, 36, 39, 44, 324, 46, 47, 69, 70, 80)
        BuffTimeIsExtendedByDeadCellsPotionStationBuff = factory.create_bool_set(False, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 121, 122, 123, 124, 257, 343)
        NurseCannotRemoveDebuff = factory.create_bool_set(False, 28, 34, 87, 89, 21, 86, 199, 332, 333, 334, 165, 146, 48, 158, 157, 350, 215, 147, 321, 43)
        AddBuffTimeAdditivelyToCap = factory.create_int_set(0, 94, 600, 383, 43200)
        MountType = factory.create_int_set(-1, 118, 6, 184, 13, 166, 11, 208, 15, 210, 16, 220, 18, 222, 19, 224, 20, 226, 21, 228, 22, 231, 24, 233, 25, 235, 26, 237, 27, 239, 28, 241, 29, 243, 30, 245, 31, 247, 32, 249, 33, 251, 34, 253, 35, 255, 36, 269, 38, 272, 39, 338, 51, 346, 53, 90, 0, 128, 1, 129, 2, 130, 3, 131, 4, 132, 5, 168, 12, 141, 7, 142, 8, 143, 9, 162, 10, 193, 14, 212, 17, 230, 23, 265, 37, 275, 40, 276, 41, 277, 42, 278, 43, 279, 44, 280, 45, 281, 46, 282, 47, 283, 48, 305, 49, 318, 50, 342, 52, 370, 54, 374, 55, 377, 56, 378, 57, 379, 58, 380, 59, 381, 60, 384, 61, 387, 62, 388, 63)



from typing import Generic

from terrex.event.context import EventContext

from .base import E, EventFilter


class AndFilter(EventFilter[E], Generic[E]):
    def __init__(self, *filters: EventFilter[E]):
        if not filters:
            raise ValueError("AndFilter requires at least one filter")

        self.filters: list[EventFilter[E]] = []

        # flatten nested AndFilter
        for f in filters:
            if isinstance(f, AndFilter):
                self.filters.extend(f.filters)
            else:
                self.filters.append(f)

        event_type = self.filters[0]._event_type
        for f in self.filters:
            if f._event_type is not event_type:
                raise TypeError("All filters in AndFilter must have the same event type")

        self._event_type = event_type

    def matches(self, ctx: EventContext) -> E | None:
        result: E | None = None
        for f in self.filters:
            result = f.matches(ctx)
            if result is None:
                return None
        return result

    def __repr__(self):
        return f"And({', '.join(repr(f) for f in self.filters)})"


class OrFilter(EventFilter[E], Generic[E]):
    def __init__(self, *filters: EventFilter[E]):
        if not filters:
            raise ValueError("OrFilter requires at least one filter")

        self.filters: list[EventFilter[E]] = []

        # flatten nested OrFilter
        for f in filters:
            if isinstance(f, OrFilter):
                self.filters.extend(f.filters)
            else:
                self.filters.append(f)

        event_type = self.filters[0]._event_type
        for f in self.filters:
            if f._event_type is not event_type:
                raise TypeError("All filters in OrFilter must have the same event type")

        self._event_type = event_type

    def matches(self, ctx: EventContext) -> E | None:
        for f in self.filters:
            result = f.matches(ctx)
            if result is not None:
                return result
        return None

    def __repr__(self):
        return f"Or({', '.join(repr(f) for f in self.filters)})"



"""
PlayerVoiceOverrideID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class PlayerVoiceOverrideID(IntEnum):
    NoOverride = 0
    Male = 1
    Female = 2
    DontStarveMale = 3
    DontStarveFemale = 4
    Cow = 5
    Chicken = 6
    Frog = 7
    Goat = 8
    Retro = 9
    Cat = 10
    Dog = 11
    Turkey = 12
    Goblin = 13
    Crow = 14
    Balloon = 15
    Undead = 16
    Vampire = 17
    Fairy = 18
    WhoopieCushion = 19
    Other = 20




from dataclasses import dataclass

from terrex.net.streamer import Reader, Writer

from .net_module import NetServerModule


@dataclass()
class NetCraftingRequestModule(NetServerModule):
    id: int = 11
    items: list[tuple[int, int]] | None = None
    chests: list[int | None] | None = None

    @classmethod
    def create(
        cls, items: list[tuple[int, int]], chests: list[int | None]
    ) -> 'NetCraftingRequestModule':
        obj = cls()
        obj.items = items
        obj.chests = chests
        return obj

    def read(self, reader: Reader) -> None:
        num_items = reader.read_7bit_encoded_int()
        self.items = []
        for _ in range(num_items):
            item_id = reader.read_int()
            stack = reader.read_7bit_encoded_int()
            self.items.append((item_id, stack))
        num_chests = reader.read_7bit_encoded_int()
        self.chests = []
        for _ in range(num_chests):
            idx = reader.read_7bit_encoded_int()
            self.chests.append(None if idx < 0 else idx)

    def write(self, writer: Writer) -> None:
        if self.items is None or self.chests is None:
            raise ValueError("items and chests must not be None")
        writer.write_7bit_encoded_int(len(self.items))
        for item_id, stack in self.items:
            writer.write_int(item_id)
            writer.write_7bit_encoded_int(stack)
        writer.write_7bit_encoded_int(len(self.chests))
        for chest in self.chests:
            idx = -1 if chest is None else chest
            writer.write_7bit_encoded_int(idx)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


# todo: can be send from server?? maybe only ClientPacket?
class OpenSignResponse(SyncPacket):
    id = MessageID.OpenSignResponse

    def __init__(
        self,
        sign_id: int = 0,
        x: int = 0,
        y: int = 0,
        text: str = "",
        player_id: int = 0,
        flags: int = 0,
    ):
        self.sign_id = sign_id
        self.x = x
        self.y = y
        self.text = text
        self.player_id = player_id
        self.flags = flags

    def read(self, reader: Reader):
        self.sign_id = reader.read_short()
        self.x = reader.read_short()
        self.y = reader.read_short()
        self.text = reader.read_dotnet_string()
        self.player_id = reader.read_byte()
        self.flags = reader.read_byte()

    def write(self, writer: Writer):
        writer.write_short(self.sign_id)
        writer.write_short(self.x)
        writer.write_short(self.y)
        writer.write_dotnet_string(self.text)
        writer.write_byte(self.player_id)
        writer.write_byte(self.flags)



import struct

from terrex.net.enum.mode import NetMode


class Reader:
    """Reading data from Terraria protocol byte buffer."""

    def __init__(
        self, data: bytes, protocol_version: int = 0, net_mode: NetMode = NetMode.SERVER
    ) -> None:
        """Initializes the reader with byte data."""
        self.data = data
        self.index = 0
        self.version = protocol_version
        self.net_mode = net_mode

    def read_byte(self) -> int:
        """Reads an unsigned 8-bit integer (byte, uint8)."""
        if self.index >= len(self.data):
            raise ValueError("Cannot read byte: no data remaining")
        res = self.data[self.index]
        self.index += 1
        return res

    def read_sbyte(self) -> int:
        """Reads a signed 8-bit integer (sbyte, int8, little-endian)."""
        if self.index + 1 > len(self.data):
            raise ValueError(
                f"Cannot read sbyte: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack("<b", self.data[self.index : self.index + 1])[0])
        self.index += 1
        return res

    def read_short(self) -> int:
        """Reads a signed 16-bit integer (short, int16, little-endian)."""
        if self.index + 2 > len(self.data):
            raise ValueError(
                f"Cannot read short: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack("<h", self.data[self.index : self.index + 2])[0])
        self.index += 2
        return res

    def read_ushort(self) -> int:
        """Reads an unsigned 16-bit integer (ushort, uint16, little-endian)."""
        if self.index + 2 > len(self.data):
            raise ValueError(
                f"Cannot read ushort: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack("<H", self.data[self.index : self.index + 2])[0])
        self.index += 2
        return res

    def read_int(self) -> int:
        """Reads a signed 32-bit integer (int, int32, little-endian)."""
        if self.index + 4 > len(self.data):
            raise ValueError(f"Cannot read int: only {len(self.data) - self.index} bytes remaining")
        res = int(struct.unpack("<i", self.data[self.index : self.index + 4])[0])
        self.index += 4
        return res

    def read_ulong(self) -> int:
        """Reads an unsigned 64-bit integer (ulong, uint64, little-endian)."""
        if self.index + 8 > len(self.data):
            raise ValueError(
                f"Cannot read ulong: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack("<Q", self.data[self.index : self.index + 8])[0])
        self.index += 8
        return res

    def read_float(self) -> float:
        """Reads a 32-bit IEEE 754 float (little-endian)."""
        if self.index + 4 > len(self.data):
            raise ValueError(
                f"Cannot read float: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack("<f", self.data[self.index : self.index + 4])[0])
        self.index += 4
        return res

    def read_single(self) -> float:
        """Reads a 32-bit IEEE 754 float (Single, little-endian)."""
        if self.index + 4 > len(self.data):
            raise ValueError(
                f"Cannot read single: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack_from("<f", self.data, self.index)[0])
        self.index += 4
        return res

    def read_double(self) -> float:
        """Reads a 64-bit IEEE 754 double (little-endian)."""
        if self.index + 8 > len(self.data):
            raise ValueError(
                f"Cannot read double: only {len(self.data) - self.index} bytes remaining"
            )
        res = int(struct.unpack("<d", self.data[self.index : self.index + 8])[0])
        self.index += 8
        return res

    def read_7bit_encoded_int(reader) -> int:
        """Reads a .NET 7-bit encoded int (variable-length)."""
        value = 0
        shift = 0

        while True:
            b = reader.read_byte()
            value |= (b & 0x7F) << shift
            if (b & 0x80) == 0:
                break
            shift += 7
        return value

    # BinaryReader.ReadString()
    def read_dotnet_string(self) -> str:
        """Reads a Terraria string: 7-bit varint length."""
        length = self.read_7bit_encoded_int()
        data = self.read_bytes(length)
        try:
            return data.decode("utf-8")
        except UnicodeDecodeError:
            return repr(data)

    def read_bytes(self, length: int) -> bytes:
        """Reads the specified number of raw bytes."""
        if self.index + length > len(self.data):
            raise ValueError("Not enough bytes remaining")
        res = self.data[self.index : self.index + length]
        self.index += length
        return res

    def remaining(self) -> bytes:
        """Returns the remaining unread bytes."""
        return self.data[self.index :]

    def eof(self) -> bool:
        """Checks if the end of data has been reached."""
        return self.index >= len(self.data)

    def read_bool(self) -> bool:
        """Reads a byte interpreted as boolean (non-zero = true)."""
        return bool(self.read_byte())


class Writer:
    """Writing data to Terraria protocol byte buffer."""

    def __init__(self, protocol_version: int = 0, net_mode: NetMode = NetMode.CLIENT) -> None:
        """Initializes the writer with an empty bytearray."""
        self.data = bytearray()
        self.version = protocol_version
        self.net_mode = net_mode

    def write_byte(self, value: int) -> None:
        """Writes an unsigned 8-bit integer (byte, uint8)."""
        self.data.append(value & 0xFF)

    def write_sbyte(self, value: int) -> None:
        """Writes a signed 8-bit integer (sbyte, int8, little-endian)."""
        self.data.extend(struct.pack("<b", value))

    def write_short(self, value: int) -> None:
        """Writes a signed 16-bit integer (short, int16, little-endian)."""
        self.data.extend(struct.pack("<h", value))

    def write_ushort(self, value: int) -> None:
        """Writes an unsigned 16-bit integer (ushort, uint16, little-endian)."""
        self.data.extend(struct.pack("<H", value))

    def write_int(self, value: int) -> None:
        """Writes a signed 32-bit integer (int, int32, little-endian)."""
        self.data.extend(struct.pack("<i", value))

    def write_ulong(self, value: int) -> None:
        """Writes an unsigned 64-bit integer (ulong, uint64, little-endian)."""
        self.data.extend(struct.pack("<Q", value))

    def write_float(self, value: float) -> None:
        """Writes a 32-bit IEEE 754 float (little-endian)."""
        self.data.extend(struct.pack("<f", value))

    def write_single(self, value: float) -> None:
        """Writes a 32-bit IEEE 754 float (Single, little-endian)."""
        self.data.extend(struct.pack("<f", value))

    def write_double(self, value: float) -> None:
        """Writes a 64-bit IEEE 754 double (little-endian)."""
        self.data.extend(struct.pack("<d", value))

    def write_7bit_encoded_int(self, value: int) -> None:
        """Writes a .NET 7-bit encoded int (variable-length)."""
        if value < 0:
            raise ValueError("7-bit encoded int must be non-negative")
        while value >= 0x80:
            self.write_byte((value & 0x7F) | 0x80)
            value >>= 7
        self.write_byte(value & 0x7F)

    def write_dotnet_string(self, text: str) -> None:
        """Writes a Terraria string: 7-bit varint length."""
        raw = text.encode("utf-8")
        self.write_7bit_encoded_int(len(raw))
        for b in raw:
            self.write_byte(b)

    def write_bytes(self, value: bytes) -> None:
        """Writes raw bytes."""
        self.data.extend(value)

    def write_bool(self, value: bool) -> None:
        """Writes a boolean as a byte (1 for true, 0 for false)."""
        self.write_byte(1 if value else 0)

    def bytes(self) -> bytes:
        """Returns the final bytes of the written data."""
        return bytes(self.data)



from dataclasses import dataclass

from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import SyncPacket


@dataclass
class SetNpcHome:
    npc_id: int
    home_tile_x: int
    home_tile_y: int
    homeless: int

    @classmethod
    def read(cls, reader: Reader) -> 'SetNpcHome':
        return cls(
            reader.read_short(),
            reader.read_short(),
            reader.read_short(),
            reader.read_byte(),
        )

    def write(self, writer: Writer) -> None:
        writer.write_short(self.npc_id)
        writer.write_short(self.home_tile_x)
        writer.write_short(self.home_tile_y)
        writer.write_byte(self.homeless)


class UpdateHomeNPC(SyncPacket):
    id = MessageID.UpdateHomeNPC

    def read(self, reader: Reader) -> None:
        self.data = SetNpcHome.read(reader)

    def write(self, writer: Writer) -> None:
        self.data.write(writer)



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer

from .base import ServerPacket


class SyncPlayerChestIndex(ServerPacket):
    id = MessageID.SyncPlayerChestIndex

    def __init__(self) -> None:
        self.player_id: int = 0
        self.chest_id: int = 0

    def read(self, reader: Reader) -> None:
        self.player_id = reader.read_byte()
        self.chest_id = reader.read_short()

    def write(self, writer: Writer) -> None:
        raise NotImplementedError(
            "Server does not send SyncPlayerChestIndex (client-bound packet only)"
        )



from terrex.id import MessageID
from terrex.packet.base import ClientPacket


class AnglerQuestFinished(ClientPacket):
    id = MessageID.AnglerQuestFinished

    def write(self, writer):
        pass



from .base import EventFilter, EventTypeFilter
from .item import ItemDrop, UpdateItemDrop, UpdateItemOwner
from .message import NewMessage
from .player import (
    BlockPlayer,
    IncomingMessage,
    ItemOwnedByMe,
    ItemOwnedByOther,
    LoginPlayer,
    OutgoingMessage,
)
from .world import UpdateTileSection

__all__ = [
    # base
    "EventFilter",
    "EventTypeFilter",
    # message
    "NewMessage",
    # player
    "OutgoingMessage",
    "IncomingMessage",
    "ItemOwnedByMe",
    "ItemOwnedByOther",
    "LoginPlayer",
    "BlockPlayer",
    # item
    "ItemDrop",
    "UpdateItemDrop",
    "UpdateItemOwner",
    # world
    "UpdateTileSection",
]



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ClientPacket


class ToggleParty(ClientPacket):
    id = MessageID.ToggleParty

    def read(self, reader: Reader) -> None:
        pass

    def write(self, writer: Writer) -> None:
        pass



from terrex.id import MessageID
from terrex.net.streamer import Reader, Writer
from terrex.packet.base import ClientPacket


class SpawnTileData(ClientPacket):
    id = MessageID.SpawnTileData

    def __init__(self, spawn_x: int = -1, spawn_y: int = -1):
        self.spawn_x = spawn_x
        self.spawn_y = spawn_y

    def write(self, writer: Writer):
        writer.write_short(self.spawn_x)
        writer.write_short(self.spawn_y)

    def read(self, reader: Reader) -> None:
        self.spawn_x = reader.read_short()
        self.spawn_y = reader.read_short()



"""
ItemUseStyleID - autogenerated from decompiled Terraria .cs files
Terraria namespace: Terraria.ID
Terraria version: v1.4.5.5
Generator: TerrariaPyGen v1.1.1 (Maksim Stashkevich)
Generated on: 2026-02-15 09:24
"""

from enum import IntEnum, auto


class ItemUseStyleID(IntEnum):
    NoneValue = 0
    Swing = 1
    EatFood = 2
    Thrust = 3
    HoldUp = 4
    Shoot = 5
    DrinkLong = 6
    DrinkOld = 7
    GolfPlay = 8
    DrinkLiquid = 9
    HiddenAnimation = 10
    MowTheLawn = 11
    Guitar = 12
    Rapier = 13
    RaiseLamp = 14
    HoldOrb = 15
    PlaySound = 16

